from __future__ import annotations

from typing import TYPE_CHECKING, Any

from ..types import MessageEntity
from .base import TelegramMethod


class SendMessageDraft(TelegramMethod[bool]):
    """
    Use this method to stream a partial message to a user while the message is being generated. Note that the streamed draft is ephemeral and acts as a temporary 30-second preview - once the output is finalized, you **must** call :class:`aiogram.methods.send_message.SendMessage` with the complete message to persist it in the user's chat. Returns :code:`True` on success.

    Source: https://core.telegram.org/bots/api#sendmessagedraft
    """

    __returning__ = bool
    __api_method__ = "sendMessageDraft"

    chat_id: int
    """Unique identifier for the target private chat"""
    draft_id: int
    """Unique identifier of the message draft; must be non-zero. Changes of drafts with the same identifier are animated."""
    message_thread_id: int | None = None
    """Unique identifier for the target message thread"""
    text: str | None = None
    """Text of the message to be sent, 0-4096 characters after entities parsing. Pass an empty text to show a 'Thinking…' placeholder."""
    parse_mode: str | None = None
    """Mode for parsing entities in the message text. See `formatting options <https://core.telegram.org/bots/api#formatting-options>`_ for more details."""
    entities: list[MessageEntity] | None = None
    """A JSON-serialized list of special entities that appear in message text, which can be specified instead of *parse_mode*"""

    if TYPE_CHECKING:
        # DO NOT EDIT MANUALLY!!!
        # This section was auto-generated via `butcher`

        def __init__(
            __pydantic__self__,
            *,
            chat_id: int,
            draft_id: int,
            message_thread_id: int | None = None,
            text: str | None = None,
            parse_mode: str | None = None,
            entities: list[MessageEntity] | None = None,
            **__pydantic_kwargs: Any,
        ) -> None:
            # DO NOT EDIT MANUALLY!!!
            # This method was auto-generated via `butcher`
            # Is needed only for type checking and IDE support without any additional plugins

            super().__init__(
                chat_id=chat_id,
                draft_id=draft_id,
                message_thread_id=message_thread_id,
                text=text,
                parse_mode=parse_mode,
                entities=entities,
                **__pydantic_kwargs,
            )



from __future__ import annotations

from typing import TYPE_CHECKING, Any

from .base import TelegramObject

if TYPE_CHECKING:
    from .web_app_info import WebAppInfo


class InlineQueryResultsButton(TelegramObject):
    """
    This object represents a button to be shown above inline query results. You **must** use exactly one of the optional fields.

    Source: https://core.telegram.org/bots/api#inlinequeryresultsbutton
    """

    text: str
    """Label text on the button"""
    web_app: WebAppInfo | None = None
    """*Optional*. Description of the `Web App <https://core.telegram.org/bots/webapps>`_ that will be launched when the user presses the button. The Web App will be able to switch back to the inline mode using the method `switchInlineQuery <https://core.telegram.org/bots/webapps#initializing-mini-apps>`_ inside the Web App."""
    start_parameter: str | None = None
    """*Optional*. `Deep-linking <https://core.telegram.org/bots/features#deep-linking>`_ parameter for the /start message sent to the bot when a user presses the button. 1-64 characters, only :code:`A-Z`, :code:`a-z`, :code:`0-9`, :code:`_` and :code:`-` are allowed."""

    if TYPE_CHECKING:
        # DO NOT EDIT MANUALLY!!!
        # This section was auto-generated via `butcher`

        def __init__(
            __pydantic__self__,
            *,
            text: str,
            web_app: WebAppInfo | None = None,
            start_parameter: str | None = None,
            **__pydantic_kwargs: Any,
        ) -> None:
            # DO NOT EDIT MANUALLY!!!
            # This method was auto-generated via `butcher`
            # Is needed only for type checking and IDE support without any additional plugins

            super().__init__(
                text=text, web_app=web_app, start_parameter=start_parameter, **__pydantic_kwargs
            )



from __future__ import annotations

from typing import TypeAlias

from .input_paid_media_live_photo import InputPaidMediaLivePhoto
from .input_paid_media_photo import InputPaidMediaPhoto
from .input_paid_media_video import InputPaidMediaVideo

InputPaidMediaUnion: TypeAlias = (
    InputPaidMediaLivePhoto | InputPaidMediaPhoto | InputPaidMediaVideo
)



from __future__ import annotations

import contextvars
from typing import TYPE_CHECKING, Any, Generic, TypeVar, cast, overload

if TYPE_CHECKING:
    from typing import Literal

__all__ = ("ContextInstanceMixin", "DataMixin")


class DataMixin:
    @property
    def data(self) -> dict[str, Any]:
        data: dict[str, Any] | None = getattr(self, "_data", None)
        if data is None:
            data = {}
            self._data = data
        return data

    def __getitem__(self, key: str) -> Any:
        return self.data[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.data[key] = value

    def __delitem__(self, key: str) -> None:
        del self.data[key]

    def __contains__(self, key: str) -> bool:
        return key in self.data

    def get(self, key: str, default: Any | None = None) -> Any | None:
        return self.data.get(key, default)


ContextInstance = TypeVar("ContextInstance")


class ContextInstanceMixin(Generic[ContextInstance]):
    __context_instance: contextvars.ContextVar[ContextInstance]

    def __init_subclass__(cls, **kwargs: Any) -> None:
        super().__init_subclass__()
        cls.__context_instance = contextvars.ContextVar(f"instance_{cls.__name__}")

    @overload
    @classmethod
    def get_current(cls) -> ContextInstance | None:  # pragma: no cover
        ...

    @overload
    @classmethod
    def get_current(
        cls,
        no_error: Literal[True],
    ) -> ContextInstance | None:  # pragma: no cover
        ...

    @overload
    @classmethod
    def get_current(
        cls,
        no_error: Literal[False],
    ) -> ContextInstance:  # pragma: no cover
        ...

    @classmethod
    def get_current(
        cls,
        no_error: bool = True,
    ) -> ContextInstance | None:  # pragma: no cover
        cls.__context_instance = cls.__context_instance

        try:
            current: ContextInstance | None = cls.__context_instance.get()
        except LookupError:
            if no_error:
                current = None
            else:
                raise

        return current

    @classmethod
    def set_current(cls, value: ContextInstance) -> contextvars.Token[ContextInstance]:
        if not isinstance(value, cls):
            msg = f"Value should be instance of {cls.__name__!r} not {type(value).__name__!r}"
            raise TypeError(msg)
        return cls.__context_instance.set(cast(ContextInstance, value))

    @classmethod
    def reset_current(cls, token: contextvars.Token[ContextInstance]) -> None:
        cls.__context_instance.reset(token)



from __future__ import annotations

from typing import TYPE_CHECKING, Any, Literal

from aiogram.enums import StoryAreaTypeType

from .story_area_type import StoryAreaType

if TYPE_CHECKING:
    from .reaction_type_union import ReactionTypeUnion


class StoryAreaTypeSuggestedReaction(StoryAreaType):
    """
    Describes a story area pointing to a suggested reaction. Currently, a story can have up to 5 suggested reaction areas.

    Source: https://core.telegram.org/bots/api#storyareatypesuggestedreaction
    """

    type: Literal[StoryAreaTypeType.SUGGESTED_REACTION] = StoryAreaTypeType.SUGGESTED_REACTION
    """Type of the area, always 'suggested_reaction'"""
    reaction_type: ReactionTypeUnion
    """Type of the reaction"""
    is_dark: bool | None = None
    """*Optional*. Pass :code:`True` if the reaction area has a dark background"""
    is_flipped: bool | None = None
    """*Optional*. Pass :code:`True` if reaction area corner is flipped"""

    if TYPE_CHECKING:
        # DO NOT EDIT MANUALLY!!!
        # This section was auto-generated via `butcher`

        def __init__(
            __pydantic__self__,
            *,
            type: Literal[
                StoryAreaTypeType.SUGGESTED_REACTION
            ] = StoryAreaTypeType.SUGGESTED_REACTION,
            reaction_type: ReactionTypeUnion,
            is_dark: bool | None = None,
            is_flipped: bool | None = None,
            **__pydantic_kwargs: Any,
        ) -> None:
            # DO NOT EDIT MANUALLY!!!
            # This method was auto-generated via `butcher`
            # Is needed only for type checking and IDE support without any additional plugins

            super().__init__(
                type=type,
                reaction_type=reaction_type,
                is_dark=is_dark,
                is_flipped=is_flipped,
                **__pydantic_kwargs,
            )



from collections.abc import Sequence
from ipaddress import IPv4Address, IPv4Network

DEFAULT_TELEGRAM_NETWORKS = [
    IPv4Network("149.154.160.0/20"),
    IPv4Network("91.108.4.0/22"),
]


class IPFilter:
    def __init__(self, ips: Sequence[str | IPv4Network | IPv4Address] | None = None):
        self._allowed_ips: set[IPv4Address] = set()

        if ips:
            self.allow(*ips)

    def allow(self, *ips: str | IPv4Network | IPv4Address) -> None:
        for ip in ips:
            self.allow_ip(ip)

    def allow_ip(self, ip: str | IPv4Network | IPv4Address) -> None:
        if isinstance(ip, str):
            ip = IPv4Network(ip) if "/" in ip else IPv4Address(ip)
        if isinstance(ip, IPv4Address):
            self._allowed_ips.add(ip)
        elif isinstance(ip, IPv4Network):
            self._allowed_ips.update(ip.hosts())
        else:
            msg = f"Invalid type of ipaddress: {type(ip)} ('{ip}')"
            raise ValueError(msg)

    @classmethod
    def default(cls) -> "IPFilter":
        return cls(DEFAULT_TELEGRAM_NETWORKS)

    def check(self, ip: str | IPv4Address) -> bool:
        if not isinstance(ip, IPv4Address):
            ip = IPv4Address(ip)
        return ip in self._allowed_ips

    def __contains__(self, item: str | IPv4Address) -> bool:
        return self.check(item)



from __future__ import annotations

from typing import TYPE_CHECKING, Any

from .base import TelegramObject

if TYPE_CHECKING:
    from .photo_size import PhotoSize


class UserProfilePhotos(TelegramObject):
    """
    This object represent a user's profile pictures.

    Source: https://core.telegram.org/bots/api#userprofilephotos
    """

    total_count: int
    """Total number of profile pictures the target user has"""
    photos: list[list[PhotoSize]]
    """Requested profile pictures (in up to 4 sizes each)"""

    if TYPE_CHECKING:
        # DO NOT EDIT MANUALLY!!!
        # This section was auto-generated via `butcher`

        def __init__(
            __pydantic__self__,
            *,
            total_count: int,
            photos: list[list[PhotoSize]],
            **__pydantic_kwargs: Any,
        ) -> None:
            # DO NOT EDIT MANUALLY!!!
            # This method was auto-generated via `butcher`
            # Is needed only for type checking and IDE support without any additional plugins

            super().__init__(total_count=total_count, photos=photos, **__pydantic_kwargs)



from __future__ import annotations

from typing import TYPE_CHECKING, Any

from .base import TelegramObject


class ShippingAddress(TelegramObject):
    """
    This object represents a shipping address.

    Source: https://core.telegram.org/bots/api#shippingaddress
    """

    country_code: str
    """Two-letter `ISO 3166-1 alpha-2 <https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2>`_ country code"""
    state: str
    """State, if applicable"""
    city: str
    """City"""
    street_line1: str
    """First line for the address"""
    street_line2: str
    """Second line for the address"""
    post_code: str
    """Address post code"""

    if TYPE_CHECKING:
        # DO NOT EDIT MANUALLY!!!
        # This section was auto-generated via `butcher`

        def __init__(
            __pydantic__self__,
            *,
            country_code: str,
            state: str,
            city: str,
            street_line1: str,
            street_line2: str,
            post_code: str,
            **__pydantic_kwargs: Any,
        ) -> None:
            # DO NOT EDIT MANUALLY!!!
            # This method was auto-generated via `butcher`
            # Is needed only for type checking and IDE support without any additional plugins

            super().__init__(
                country_code=country_code,
                state=state,
                city=city,
                street_line1=street_line1,
                street_line2=street_line2,
                post_code=post_code,
                **__pydantic_kwargs,
            )



from __future__ import annotations

from typing import TYPE_CHECKING, Any

from .base import TelegramObject

if TYPE_CHECKING:
    from .passport_file import PassportFile


class EncryptedPassportElement(TelegramObject):
    """
    Describes documents or other Telegram Passport elements shared with the bot by the user.

    Source: https://core.telegram.org/bots/api#encryptedpassportelement
    """

    type: str
    """Element type. One of 'personal_details', 'passport', 'driver_license', 'identity_card', 'internal_passport', 'address', 'utility_bill', 'bank_statement', 'rental_agreement', 'passport_registration', 'temporary_registration', 'phone_number', 'email'."""
    hash: str
    """Base64-encoded element hash for using in :class:`aiogram.types.passport_element_error_unspecified.PassportElementErrorUnspecified`"""
    data: str | None = None
    """*Optional*. Base64-encoded encrypted Telegram Passport element data provided by the user; available only for 'personal_details', 'passport', 'driver_license', 'identity_card', 'internal_passport' and 'address' types. Can be decrypted and verified using the accompanying :class:`aiogram.types.encrypted_credentials.EncryptedCredentials`."""
    phone_number: str | None = None
    """*Optional*. User's verified phone number; available only for 'phone_number' type"""
    email: str | None = None
    """*Optional*. User's verified email address; available only for 'email' type"""
    files: list[PassportFile] | None = None
    """*Optional*. Array of encrypted files with documents provided by the user; available only for 'utility_bill', 'bank_statement', 'rental_agreement', 'passport_registration' and 'temporary_registration' types. Files can be decrypted and verified using the accompanying :class:`aiogram.types.encrypted_credentials.EncryptedCredentials`."""
    front_side: PassportFile | None = None
    """*Optional*. Encrypted file with the front side of the document, provided by the user; available only for 'passport', 'driver_license', 'identity_card' and 'internal_passport'. The file can be decrypted and verified using the accompanying :class:`aiogram.types.encrypted_credentials.EncryptedCredentials`."""
    reverse_side: PassportFile | None = None
    """*Optional*. Encrypted file with the reverse side of the document, provided by the user; available only for 'driver_license' and 'identity_card'. The file can be decrypted and verified using the accompanying :class:`aiogram.types.encrypted_credentials.EncryptedCredentials`."""
    selfie: PassportFile | None = None
    """*Optional*. Encrypted file with the selfie of the user holding a document, provided by the user; available if requested for 'passport', 'driver_license', 'identity_card' and 'internal_passport'. The file can be decrypted and verified using the accompanying :class:`aiogram.types.encrypted_credentials.EncryptedCredentials`."""
    translation: list[PassportFile] | None = None
    """*Optional*. Array of encrypted files with translated versions of documents provided by the user; available if requested for 'passport', 'driver_license', 'identity_card', 'internal_passport', 'utility_bill', 'bank_statement', 'rental_agreement', 'passport_registration' and 'temporary_registration' types. Files can be decrypted and verified using the accompanying :class:`aiogram.types.encrypted_credentials.EncryptedCredentials`."""

    if TYPE_CHECKING:
        # DO NOT EDIT MANUALLY!!!
        # This section was auto-generated via `butcher`

        def __init__(
            __pydantic__self__,
            *,
            type: str,
            hash: str,
            data: str | None = None,
            phone_number: str | None = None,
            email: str | None = None,
            files: list[PassportFile] | None = None,
            front_side: PassportFile | None = None,
            reverse_side: PassportFile | None = None,
            selfie: PassportFile | None = None,
            translation: list[PassportFile] | None = None,
            **__pydantic_kwargs: Any,
        ) -> None:
            # DO NOT EDIT MANUALLY!!!
            # This method was auto-generated via `butcher`
            # Is needed only for type checking and IDE support without any additional plugins

            super().__init__(
                type=type,
                hash=hash,
                data=data,
                phone_number=phone_number,
                email=email,
                files=files,
                front_side=front_side,
                reverse_side=reverse_side,
                selfie=selfie,
                translation=translation,
                **__pydantic_kwargs,
            )



from __future__ import annotations

from typing import TYPE_CHECKING, Any

from ..types import Sticker
from .base import TelegramMethod


class GetCustomEmojiStickers(TelegramMethod[list[Sticker]]):
    """
    Use this method to get information about custom emoji stickers by their identifiers. Returns an Array of :class:`aiogram.types.sticker.Sticker` objects.

    Source: https://core.telegram.org/bots/api#getcustomemojistickers
    """

    __returning__ = list[Sticker]
    __api_method__ = "getCustomEmojiStickers"

    custom_emoji_ids: list[str]
    """A JSON-serialized list of custom emoji identifiers. At most 200 custom emoji identifiers can be specified."""

    if TYPE_CHECKING:
        # DO NOT EDIT MANUALLY!!!
        # This section was auto-generated via `butcher`

        def __init__(
            __pydantic__self__, *, custom_emoji_ids: list[str], **__pydantic_kwargs: Any
        ) -> None:
            # DO NOT EDIT MANUALLY!!!
            # This method was auto-generated via `butcher`
            # Is needed only for type checking and IDE support without any additional plugins

            super().__init__(custom_emoji_ids=custom_emoji_ids, **__pydantic_kwargs)



from __future__ import annotations

from typing import TYPE_CHECKING, Any

from .base import TelegramObject

if TYPE_CHECKING:
    from .chat import Chat
    from .message_entity import MessageEntity
    from .user import User


class ChecklistTask(TelegramObject):
    """
    Describes a task in a checklist.

    Source: https://core.telegram.org/bots/api#checklisttask
    """

    id: int
    """Unique identifier of the task"""
    text: str
    """Text of the task"""
    text_entities: list[MessageEntity] | None = None
    """*Optional*. Special entities that appear in the task text"""
    completed_by_user: User | None = None
    """*Optional*. User that completed the task; omitted if the task wasn't completed by a user"""
    completed_by_chat: Chat | None = None
    """*Optional*. Chat that completed the task; omitted if the task wasn't completed by a chat"""
    completion_date: int | None = None
    """*Optional*. Point in time (Unix timestamp) when the task was completed; 0 if the task wasn't completed"""

    if TYPE_CHECKING:
        # DO NOT EDIT MANUALLY!!!
        # This section was auto-generated via `butcher`

        def __init__(
            __pydantic__self__,
            *,
            id: int,
            text: str,
            text_entities: list[MessageEntity] | None = None,
            completed_by_user: User | None = None,
            completed_by_chat: Chat | None = None,
            completion_date: int | None = None,
            **__pydantic_kwargs: Any,
        ) -> None:
            # DO NOT EDIT MANUALLY!!!
            # This method was auto-generated via `butcher`
            # Is needed only for type checking and IDE support without any additional plugins

            super().__init__(
                id=id,
                text=text,
                text_entities=text_entities,
                completed_by_user=completed_by_user,
                completed_by_chat=completed_by_chat,
                completion_date=completion_date,
                **__pydantic_kwargs,
            )



from __future__ import annotations

from typing import TYPE_CHECKING, Any

from .base import TelegramObject
from .custom import DateTime


class VideoChatScheduled(TelegramObject):
    """
    This object represents a service message about a video chat scheduled in the chat.

    Source: https://core.telegram.org/bots/api#videochatscheduled
    """

    start_date: DateTime
    """Point in time (Unix timestamp) when the video chat is supposed to be started by a chat administrator"""

    if TYPE_CHECKING:
        # DO NOT EDIT MANUALLY!!!
        # This section was auto-generated via `butcher`

        def __init__(
            __pydantic__self__, *, start_date: DateTime, **__pydantic_kwargs: Any
        ) -> None:
            # DO NOT EDIT MANUALLY!!!
            # This method was auto-generated via `butcher`
            # Is needed only for type checking and IDE support without any additional plugins

            super().__init__(start_date=start_date, **__pydantic_kwargs)



from enum import Enum


class MessageEntityType(str, Enum):
    """
    This object represents type of message entity

    Source: https://core.telegram.org/bots/api#messageentity
    """

    MENTION = "mention"
    HASHTAG = "hashtag"
    CASHTAG = "cashtag"
    BOT_COMMAND = "bot_command"
    URL = "url"
    EMAIL = "email"
    PHONE_NUMBER = "phone_number"
    BOLD = "bold"
    ITALIC = "italic"
    UNDERLINE = "underline"
    STRIKETHROUGH = "strikethrough"
    SPOILER = "spoiler"
    BLOCKQUOTE = "blockquote"
    EXPANDABLE_BLOCKQUOTE = "expandable_blockquote"
    CODE = "code"
    PRE = "pre"
    TEXT_LINK = "text_link"
    TEXT_MENTION = "text_mention"
    CUSTOM_EMOJI = "custom_emoji"
    DATE_TIME = "date_time"



from __future__ import annotations

from .base import TelegramObject


class CallbackGame(TelegramObject):
    """
    A placeholder, currently holds no information. Use `BotFather <https://t.me/botfather>`_ to set up your game.

    Source: https://core.telegram.org/bots/api#callbackgame
    """



from __future__ import annotations

from typing import TYPE_CHECKING, Any

from .base import TelegramObject

if TYPE_CHECKING:
    from .message import Message


class SuggestedPostDeclined(TelegramObject):
    """
    Describes a service message about the rejection of a suggested post.

    Source: https://core.telegram.org/bots/api#suggestedpostdeclined
    """

    suggested_post_message: Message | None = None
    """*Optional*. Message containing the suggested post. Note that the :class:`aiogram.types.message.Message` object in this field will not contain the *reply_to_message* field even if it itself is a reply."""
    comment: str | None = None
    """*Optional*. Comment with which the post was declined"""

    if TYPE_CHECKING:
        # DO NOT EDIT MANUALLY!!!
        # This section was auto-generated via `butcher`

        def __init__(
            __pydantic__self__,
            *,
            suggested_post_message: Message | None = None,
            comment: str | None = None,
            **__pydantic_kwargs: Any,
        ) -> None:
            # DO NOT EDIT MANUALLY!!!
            # This method was auto-generated via `butcher`
            # Is needed only for type checking and IDE support without any additional plugins

            super().__init__(
                suggested_post_message=suggested_post_message, comment=comment, **__pydantic_kwargs
            )



from __future__ import annotations

from typing import TypeAlias

from .input_media_animation import InputMediaAnimation
from .input_media_audio import InputMediaAudio
from .input_media_document import InputMediaDocument
from .input_media_live_photo import InputMediaLivePhoto
from .input_media_photo import InputMediaPhoto
from .input_media_video import InputMediaVideo

InputMediaUnion: TypeAlias = (
    InputMediaAnimation
    | InputMediaAudio
    | InputMediaDocument
    | InputMediaLivePhoto
    | InputMediaPhoto
    | InputMediaVideo
)



from __future__ import annotations

from typing import TYPE_CHECKING, TypedDict

from typing_extensions import NotRequired

if TYPE_CHECKING:
    from aiogram import Bot, Dispatcher, Router
    from aiogram.dispatcher.event.handler import HandlerObject
    from aiogram.dispatcher.middlewares.user_context import EventContext
    from aiogram.fsm.context import FSMContext
    from aiogram.fsm.storage.base import BaseStorage
    from aiogram.types import Chat, Update, User
    from aiogram.utils.i18n import I18n, I18nMiddleware


class DispatcherData(TypedDict, total=False):
    """
    Dispatcher and bot related data.
    """

    dispatcher: Dispatcher
    """Instance of the Dispatcher from which the handler was called."""
    bot: Bot
    """Bot that received the update."""
    bots: NotRequired[list[Bot]]
    """List of all bots in the Dispatcher. Used only in polling mode."""
    event_update: Update
    """Update object that triggered the handler."""
    event_router: Router
    """Router that was used to find the handler."""
    handler: NotRequired[HandlerObject]
    """Handler object that was called.
    Available only in the handler itself and inner middlewares."""


class UserContextData(TypedDict, total=False):
    """
    Event context related data about user and chat.
    """

    event_context: EventContext
    """Event context object that contains user and chat data."""
    event_from_user: NotRequired[User]
    """User object that triggered the handler."""
    event_chat: NotRequired[Chat]
    """Chat object that triggered the handler.
    .. deprecated:: 3.5.0
        Use :attr:`event_context.chat` instead."""
    event_thread_id: NotRequired[int]
    """Thread ID of the chat that triggered the handler.
    .. deprecated:: 3.5.0
        Use :attr:`event_context.chat` instead."""
    event_business_connection_id: NotRequired[str]
    """Business connection ID of the chat that triggered the handler.
    .. deprecated:: 3.5.0
        Use :attr:`event_context.business_connection_id` instead."""


class FSMData(TypedDict, total=False):
    """
    FSM related data.
    """

    fsm_storage: BaseStorage
    """Storage used for FSM."""
    state: NotRequired[FSMContext]
    """Current state of the FSM."""
    raw_state: NotRequired[str | None]
    """Raw state of the FSM."""


class I18nData(TypedDict, total=False):
    """
    I18n related data.

    Is not included by default, you need to add it to your own Data class if you need it.
    """

    i18n: I18n
    """I18n object."""
    i18n_middleware: I18nMiddleware
    """I18n middleware."""


class MiddlewareData(
    DispatcherData,
    UserContextData,
    FSMData,
    # I18nData, # Disabled by default, add it if you need it to your own Data class.
    total=False,
):
    """
    Data passed to the handler by the middlewares.

    You can add your own data by extending this class.
    """



from __future__ import annotations

from .base import MutableTelegramObject


class PassportElementError(MutableTelegramObject):
    """
    This object represents an error in the Telegram Passport element which was submitted that should be resolved by the user. It should be one of:

     - :class:`aiogram.types.passport_element_error_data_field.PassportElementErrorDataField`
     - :class:`aiogram.types.passport_element_error_front_side.PassportElementErrorFrontSide`
     - :class:`aiogram.types.passport_element_error_reverse_side.PassportElementErrorReverseSide`
     - :class:`aiogram.types.passport_element_error_selfie.PassportElementErrorSelfie`
     - :class:`aiogram.types.passport_element_error_file.PassportElementErrorFile`
     - :class:`aiogram.types.passport_element_error_files.PassportElementErrorFiles`
     - :class:`aiogram.types.passport_element_error_translation_file.PassportElementErrorTranslationFile`
     - :class:`aiogram.types.passport_element_error_translation_files.PassportElementErrorTranslationFiles`
     - :class:`aiogram.types.passport_element_error_unspecified.PassportElementErrorUnspecified`

    Source: https://core.telegram.org/bots/api#passportelementerror
    """



from __future__ import annotations

from typing import TYPE_CHECKING, Any, Literal

from ..enums import ChatMemberStatus
from .chat_member import ChatMember
from .custom import DateTime

if TYPE_CHECKING:
    from .user import User


class ChatMemberRestricted(ChatMember):
    """
    Represents a `chat member <https://core.telegram.org/bots/api#chatmember>`_ that is under certain restrictions in the chat. Supergroups only.

    Source: https://core.telegram.org/bots/api#chatmemberrestricted
    """

    status: Literal[ChatMemberStatus.RESTRICTED] = ChatMemberStatus.RESTRICTED
    """The member's status in the chat, always 'restricted'"""
    user: User
    """Information about the user"""
    is_member: bool
    """:code:`True`, if the user is a member of the chat at the moment of the request"""
    can_send_messages: bool
    """:code:`True`, if the user is allowed to send text messages, contacts, giveaways, giveaway winners, invoices, locations and venues"""
    can_send_audios: bool
    """:code:`True`, if the user is allowed to send audios"""
    can_send_documents: bool
    """:code:`True`, if the user is allowed to send documents"""
    can_send_photos: bool
    """:code:`True`, if the user is allowed to send photos"""
    can_send_videos: bool
    """:code:`True`, if the user is allowed to send videos"""
    can_send_video_notes: bool
    """:code:`True`, if the user is allowed to send video notes"""
    can_send_voice_notes: bool
    """:code:`True`, if the user is allowed to send voice notes"""
    can_send_polls: bool
    """:code:`True`, if the user is allowed to send polls and checklists"""
    can_send_other_messages: bool
    """:code:`True`, if the user is allowed to send animations, games, stickers and use inline bots"""
    can_add_web_page_previews: bool
    """:code:`True`, if the user is allowed to add web page previews to their messages"""
    can_react_to_messages: bool
    """:code:`True`, if the user is allowed to react to messages"""
    can_edit_tag: bool
    """:code:`True`, if the user is allowed to edit their own tag"""
    can_change_info: bool
    """:code:`True`, if the user is allowed to change the chat title, photo and other settings"""
    can_invite_users: bool
    """:code:`True`, if the user is allowed to invite new users to the chat"""
    can_pin_messages: bool
    """:code:`True`, if the user is allowed to pin messages"""
    can_manage_topics: bool
    """:code:`True`, if the user is allowed to create forum topics"""
    until_date: DateTime
    """Date when restrictions will be lifted for this user; Unix time. If 0, then the user is restricted forever"""
    tag: str | None = None
    """*Optional*. Tag of the member"""

    if TYPE_CHECKING:
        # DO NOT EDIT MANUALLY!!!
        # This section was auto-generated via `butcher`

        def __init__(
            __pydantic__self__,
            *,
            status: Literal[ChatMemberStatus.RESTRICTED] = ChatMemberStatus.RESTRICTED,
            user: User,
            is_member: bool,
            can_send_messages: bool,
            can_send_audios: bool,
            can_send_documents: bool,
            can_send_photos: bool,
            can_send_videos: bool,
            can_send_video_notes: bool,
            can_send_voice_notes: bool,
            can_send_polls: bool,
            can_send_other_messages: bool,
            can_add_web_page_previews: bool,
            can_react_to_messages: bool,
            can_edit_tag: bool,
            can_change_info: bool,
            can_invite_users: bool,
            can_pin_messages: bool,
            can_manage_topics: bool,
            until_date: DateTime,
            tag: str | None = None,
            **__pydantic_kwargs: Any,
        ) -> None:
            # DO NOT EDIT MANUALLY!!!
            # This method was auto-generated via `butcher`
            # Is needed only for type checking and IDE support without any additional plugins

            super().__init__(
                status=status,
                user=user,
                is_member=is_member,
                can_send_messages=can_send_messages,
                can_send_audios=can_send_audios,
                can_send_documents=can_send_documents,
                can_send_photos=can_send_photos,
                can_send_videos=can_send_videos,
                can_send_video_notes=can_send_video_notes,
                can_send_voice_notes=can_send_voice_notes,
                can_send_polls=can_send_polls,
                can_send_other_messages=can_send_other_messages,
                can_add_web_page_previews=can_add_web_page_previews,
                can_react_to_messages=can_react_to_messages,
                can_edit_tag=can_edit_tag,
                can_change_info=can_change_info,
                can_invite_users=can_invite_users,
                can_pin_messages=can_pin_messages,
                can_manage_topics=can_manage_topics,
                until_date=until_date,
                tag=tag,
                **__pydantic_kwargs,
            )



from __future__ import annotations

from typing import TYPE_CHECKING, Any, Literal

from ..enums import TransactionPartnerType
from .transaction_partner import TransactionPartner

if TYPE_CHECKING:
    from .chat import Chat
    from .gift import Gift


class TransactionPartnerChat(TransactionPartner):
    """
    Describes a transaction with a chat.

    Source: https://core.telegram.org/bots/api#transactionpartnerchat
    """

    type: Literal[TransactionPartnerType.CHAT] = TransactionPartnerType.CHAT
    """Type of the transaction partner, always 'chat'"""
    chat: Chat
    """Information about the chat"""
    gift: Gift | None = None
    """*Optional*. The gift sent to the chat by the bot"""

    if TYPE_CHECKING:
        # DO NOT EDIT MANUALLY!!!
        # This section was auto-generated via `butcher`

        def __init__(
            __pydantic__self__,
            *,
            type: Literal[TransactionPartnerType.CHAT] = TransactionPartnerType.CHAT,
            chat: Chat,
            gift: Gift | None = None,
            **__pydantic_kwargs: Any,
        ) -> None:
            # DO NOT EDIT MANUALLY!!!
            # This method was auto-generated via `butcher`
            # Is needed only for type checking and IDE support without any additional plugins

            super().__init__(type=type, chat=chat, gift=gift, **__pydantic_kwargs)



from __future__ import annotations

from typing import TYPE_CHECKING, Any

from .base import TelegramObject


class Birthdate(TelegramObject):
    """
    Describes the birthdate of a user.

    Source: https://core.telegram.org/bots/api#birthdate
    """

    day: int
    """Day of the user's birth; 1-31"""
    month: int
    """Month of the user's birth; 1-12"""
    year: int | None = None
    """*Optional*. Year of the user's birth"""

    if TYPE_CHECKING:
        # DO NOT EDIT MANUALLY!!!
        # This section was auto-generated via `butcher`

        def __init__(
            __pydantic__self__,
            *,
            day: int,
            month: int,
            year: int | None = None,
            **__pydantic_kwargs: Any,
        ) -> None:
            # DO NOT EDIT MANUALLY!!!
            # This method was auto-generated via `butcher`
            # Is needed only for type checking and IDE support without any additional plugins

            super().__init__(day=day, month=month, year=year, **__pydantic_kwargs)



from __future__ import annotations

from .base import TelegramObject


class InputStoryContent(TelegramObject):
    """
    This object describes the content of a story to post. Currently, it can be one of

     - :class:`aiogram.types.input_story_content_photo.InputStoryContentPhoto`
     - :class:`aiogram.types.input_story_content_video.InputStoryContentVideo`

    Source: https://core.telegram.org/bots/api#inputstorycontent
    """



from __future__ import annotations

from typing import TYPE_CHECKING, Any

from .base import MutableTelegramObject

if TYPE_CHECKING:
    from .web_app_info import WebAppInfo


class MenuButton(MutableTelegramObject):
    """
    This object describes the bot's menu button in a private chat. It should be one of

     - :class:`aiogram.types.menu_button_commands.MenuButtonCommands`
     - :class:`aiogram.types.menu_button_web_app.MenuButtonWebApp`
     - :class:`aiogram.types.menu_button_default.MenuButtonDefault`

    If a menu button other than :class:`aiogram.types.menu_button_default.MenuButtonDefault` is set for a private chat, then it is applied in the chat. Otherwise the default menu button is applied. By default, the menu button opens the list of bot commands.

    Source: https://core.telegram.org/bots/api#menubutton
    """

    type: str
    """Type of the button"""
    text: str | None = None
    """*Optional*. Text on the button"""
    web_app: WebAppInfo | None = None
    """*Optional*. Description of the Web App that will be launched when the user presses the button. The Web App will be able to send an arbitrary message on behalf of the user using the method :class:`aiogram.methods.answer_web_app_query.AnswerWebAppQuery`. Alternatively, a :code:`t.me` link to a Web App of the bot can be specified in the object instead of the Web App's URL, in which case the Web App will be opened as if the user pressed the link."""

    if TYPE_CHECKING:
        # DO NOT EDIT MANUALLY!!!
        # This section was auto-generated via `butcher`

        def __init__(
            __pydantic__self__,
            *,
            type: str,
            text: str | None = None,
            web_app: WebAppInfo | None = None,
            **__pydantic_kwargs: Any,
        ) -> None:
            # DO NOT EDIT MANUALLY!!!
            # This method was auto-generated via `butcher`
            # Is needed only for type checking and IDE support without any additional plugins

            super().__init__(type=type, text=text, web_app=web_app, **__pydantic_kwargs)



from typing import TYPE_CHECKING, Any

from ..types import BotName
from .base import TelegramMethod


class GetMyName(TelegramMethod[BotName]):
    """
    Use this method to get the current bot name for the given user language. Returns :class:`aiogram.types.bot_name.BotName` on success.

    Source: https://core.telegram.org/bots/api#getmyname
    """

    __returning__ = BotName
    __api_method__ = "getMyName"

    language_code: str | None = None
    """A two-letter ISO 639-1 language code or an empty string"""

    if TYPE_CHECKING:
        # DO NOT EDIT MANUALLY!!!
        # This section was auto-generated via `butcher`

        def __init__(
            __pydantic__self__, *, language_code: str | None = None, **__pydantic_kwargs: Any
        ) -> None:
            # DO NOT EDIT MANUALLY!!!
            # This method was auto-generated via `butcher`
            # Is needed only for type checking and IDE support without any additional plugins

            super().__init__(language_code=language_code, **__pydantic_kwargs)



from __future__ import annotations

from typing import TYPE_CHECKING, Any

from .base import TelegramObject

if TYPE_CHECKING:
    from .message import Message
    from .star_amount import StarAmount


class SuggestedPostPaid(TelegramObject):
    """
    Describes a service message about a successful payment for a suggested post.

    Source: https://core.telegram.org/bots/api#suggestedpostpaid
    """

    currency: str
    """Currency in which the payment was made. Currently, one of 'XTR' for Telegram Stars or 'TON' for toncoins"""
    suggested_post_message: Message | None = None
    """*Optional*. Message containing the suggested post. Note that the :class:`aiogram.types.message.Message` object in this field will not contain the *reply_to_message* field even if it itself is a reply."""
    amount: int | None = None
    """*Optional*. The amount of the currency that was received by the channel in nanotoncoins; for payments in toncoins only"""
    star_amount: StarAmount | None = None
    """*Optional*. The amount of Telegram Stars that was received by the channel; for payments in Telegram Stars only"""

    if TYPE_CHECKING:
        # DO NOT EDIT MANUALLY!!!
        # This section was auto-generated via `butcher`

        def __init__(
            __pydantic__self__,
            *,
            currency: str,
            suggested_post_message: Message | None = None,
            amount: int | None = None,
            star_amount: StarAmount | None = None,
            **__pydantic_kwargs: Any,
        ) -> None:
            # DO NOT EDIT MANUALLY!!!
            # This method was auto-generated via `butcher`
            # Is needed only for type checking and IDE support without any additional plugins

            super().__init__(
                currency=currency,
                suggested_post_message=suggested_post_message,
                amount=amount,
                star_amount=star_amount,
                **__pydantic_kwargs,
            )



from __future__ import annotations

from typing import TYPE_CHECKING, Any

from ..types import InlineQueryResultUnion, PreparedInlineMessage
from .base import TelegramMethod


class SavePreparedInlineMessage(TelegramMethod[PreparedInlineMessage]):
    """
    Stores a message that can be sent by a user of a Mini App. Returns a :class:`aiogram.types.prepared_inline_message.PreparedInlineMessage` object.

    Source: https://core.telegram.org/bots/api#savepreparedinlinemessage
    """

    __returning__ = PreparedInlineMessage
    __api_method__ = "savePreparedInlineMessage"

    user_id: int
    """Unique identifier of the target user that can use the prepared message"""
    result: InlineQueryResultUnion
    """A JSON-serialized object describing the message to be sent"""
    allow_user_chats: bool | None = None
    """Pass :code:`True` if the message can be sent to private chats with users"""
    allow_bot_chats: bool | None = None
    """Pass :code:`True` if the message can be sent to private chats with bots"""
    allow_group_chats: bool | None = None
    """Pass :code:`True` if the message can be sent to group and supergroup chats"""
    allow_channel_chats: bool | None = None
    """Pass :code:`True` if the message can be sent to channel chats"""

    if TYPE_CHECKING:
        # DO NOT EDIT MANUALLY!!!
        # This section was auto-generated via `butcher`

        def __init__(
            __pydantic__self__,
            *,
            user_id: int,
            result: InlineQueryResultUnion,
            allow_user_chats: bool | None = None,
            allow_bot_chats: bool | None = None,
            allow_group_chats: bool | None = None,
            allow_channel_chats: bool | None = None,
            **__pydantic_kwargs: Any,
        ) -> None:
            # DO NOT EDIT MANUALLY!!!
            # This method was auto-generated via `butcher`
            # Is needed only for type checking and IDE support without any additional plugins

            super().__init__(
                user_id=user_id,
                result=result,
                allow_user_chats=allow_user_chats,
                allow_bot_chats=allow_bot_chats,
                allow_group_chats=allow_group_chats,
                allow_channel_chats=allow_channel_chats,
                **__pydantic_kwargs,
            )



from abc import ABC

from aiogram.handlers import BaseHandler
from aiogram.types import Poll, PollOption


class PollHandler(BaseHandler[Poll], ABC):
    """
    Base class for poll handlers
    """

    @property
    def question(self) -> str:
        return self.event.question

    @property
    def options(self) -> list[PollOption]:
        return self.event.options



from typing import TYPE_CHECKING, Any

from ..types import ChatIdUnion, UserChatBoosts
from .base import TelegramMethod


class GetUserChatBoosts(TelegramMethod[UserChatBoosts]):
    """
    Use this method to get the list of boosts added to a chat by a user. Requires administrator rights in the chat. Returns a :class:`aiogram.types.user_chat_boosts.UserChatBoosts` object.

    Source: https://core.telegram.org/bots/api#getuserchatboosts
    """

    __returning__ = UserChatBoosts
    __api_method__ = "getUserChatBoosts"

    chat_id: ChatIdUnion
    """Unique identifier for the chat or username of the channel in the format :code:`@username`"""
    user_id: int
    """Unique identifier of the target user"""

    if TYPE_CHECKING:
        # DO NOT EDIT MANUALLY!!!
        # This section was auto-generated via `butcher`

        def __init__(
            __pydantic__self__, *, chat_id: ChatIdUnion, user_id: int, **__pydantic_kwargs: Any
        ) -> None:
            # DO NOT EDIT MANUALLY!!!
            # This method was auto-generated via `butcher`
            # Is needed only for type checking and IDE support without any additional plugins

            super().__init__(chat_id=chat_id, user_id=user_id, **__pydantic_kwargs)



from __future__ import annotations

from typing import TYPE_CHECKING, Any, Literal

from ..enums import RevenueWithdrawalStateType
from .custom import DateTime
from .revenue_withdrawal_state import RevenueWithdrawalState


class RevenueWithdrawalStateSucceeded(RevenueWithdrawalState):
    """
    The withdrawal succeeded.

    Source: https://core.telegram.org/bots/api#revenuewithdrawalstatesucceeded
    """

    type: Literal[RevenueWithdrawalStateType.SUCCEEDED] = RevenueWithdrawalStateType.SUCCEEDED
    """Type of the state, always 'succeeded'"""
    date: DateTime
    """Date the withdrawal was completed in Unix time"""
    url: str
    """An HTTPS URL that can be used to see transaction details"""

    if TYPE_CHECKING:
        # DO NOT EDIT MANUALLY!!!
        # This section was auto-generated via `butcher`

        def __init__(
            __pydantic__self__,
            *,
            type: Literal[
                RevenueWithdrawalStateType.SUCCEEDED
            ] = RevenueWithdrawalStateType.SUCCEEDED,
            date: DateTime,
            url: str,
            **__pydantic_kwargs: Any,
        ) -> None:
            # DO NOT EDIT MANUALLY!!!
            # This method was auto-generated via `butcher`
            # Is needed only for type checking and IDE support without any additional plugins

            super().__init__(type=type, date=date, url=url, **__pydantic_kwargs)



from __future__ import annotations

from typing import TYPE_CHECKING, Any

from ..types import StarTransactions
from .base import TelegramMethod


class GetStarTransactions(TelegramMethod[StarTransactions]):
    """
    Returns the bot's Telegram Star transactions in chronological order. On success, returns a :class:`aiogram.types.star_transactions.StarTransactions` object.

    Source: https://core.telegram.org/bots/api#getstartransactions
    """

    __returning__ = StarTransactions
    __api_method__ = "getStarTransactions"

    offset: int | None = None
    """Number of transactions to skip in the response"""
    limit: int | None = None
    """The maximum number of transactions to be retrieved. Values between 1-100 are accepted. Defaults to 100."""

    if TYPE_CHECKING:
        # DO NOT EDIT MANUALLY!!!
        # This section was auto-generated via `butcher`

        def __init__(
            __pydantic__self__,
            *,
            offset: int | None = None,
            limit: int | None = None,
            **__pydantic_kwargs: Any,
        ) -> None:
            # DO NOT EDIT MANUALLY!!!
            # This method was auto-generated via `butcher`
            # Is needed only for type checking and IDE support without any additional plugins

            super().__init__(offset=offset, limit=limit, **__pydantic_kwargs)



from __future__ import annotations

from typing import TYPE_CHECKING, Any, Literal

from ..client.default import Default
from ..enums import InputMediaType
from .input_media import InputMedia
from .input_poll_option_media import InputPollOptionMedia

if TYPE_CHECKING:
    from .input_file_union import InputFileUnion
    from .message_entity import MessageEntity


class InputMediaPhoto(InputMedia, InputPollOptionMedia):
    """
    Represents a photo to be sent.

    Source: https://core.telegram.org/bots/api#inputmediaphoto
    """

    type: Literal[InputMediaType.PHOTO] = InputMediaType.PHOTO
    """Type of the result, must be *photo*"""
    media: InputFileUnion
    """File to send. Pass a file_id to send a file that exists on the Telegram servers (recommended), pass an HTTP URL for Telegram to get a file from the Internet, or pass 'attach://<file_attach_name>' to upload a new one using multipart/form-data under <file_attach_name> name. :ref:`More information on Sending Files » <sending-files>`"""
    caption: str | None = None
    """*Optional*. Caption of the photo to be sent, 0-1024 characters after entities parsing"""
    parse_mode: str | Default | None = Default("parse_mode")
    """*Optional*. Mode for parsing entities in the photo caption. See `formatting options <https://core.telegram.org/bots/api#formatting-options>`_ for more details."""
    caption_entities: list[MessageEntity] | None = None
    """*Optional*. List of special entities that appear in the caption, which can be specified instead of *parse_mode*"""
    show_caption_above_media: bool | Default | None = Default("show_caption_above_media")
    """*Optional*. Pass :code:`True`, if the caption must be shown above the message media"""
    has_spoiler: bool | None = None
    """*Optional*. Pass :code:`True` if the photo needs to be covered with a spoiler animation"""

    if TYPE_CHECKING:
        # DO NOT EDIT MANUALLY!!!
        # This section was auto-generated via `butcher`

        def __init__(
            __pydantic__self__,
            *,
            type: Literal[InputMediaType.PHOTO] = InputMediaType.PHOTO,
            media: InputFileUnion,
            caption: str | None = None,
            parse_mode: str | Default | None = Default("parse_mode"),
            caption_entities: list[MessageEntity] | None = None,
            show_caption_above_media: bool | Default | None = Default("show_caption_above_media"),
            has_spoiler: bool | None = None,
            **__pydantic_kwargs: Any,
        ) -> None:
            # DO NOT EDIT MANUALLY!!!
            # This method was auto-generated via `butcher`
            # Is needed only for type checking and IDE support without any additional plugins

            super().__init__(
                type=type,
                media=media,
                caption=caption,
                parse_mode=parse_mode,
                caption_entities=caption_entities,
                show_caption_above_media=show_caption_above_media,
                has_spoiler=has_spoiler,
                **__pydantic_kwargs,
            )



from typing import Any

from .text_decorations import html_decoration, markdown_decoration


def _join(*content: Any, sep: str = " ") -> str:
    return sep.join(map(str, content))


def text(*content: Any, sep: str = " ") -> str:
    """
    Join all elements with a separator

    :param content:
    :param sep:
    :return:
    """
    return _join(*content, sep=sep)


def bold(*content: Any, sep: str = " ") -> str:
    """
    Make bold text (Markdown)

    :param content:
    :param sep:
    :return:
    """
    return markdown_decoration.bold(value=markdown_decoration.quote(_join(*content, sep=sep)))


def hbold(*content: Any, sep: str = " ") -> str:
    """
    Make bold text (HTML)

    :param content:
    :param sep:
    :return:
    """
    return html_decoration.bold(value=html_decoration.quote(_join(*content, sep=sep)))


def italic(*content: Any, sep: str = " ") -> str:
    """
    Make italic text (Markdown)

    :param content:
    :param sep:
    :return:
    """
    return markdown_decoration.italic(value=markdown_decoration.quote(_join(*content, sep=sep)))


def hitalic(*content: Any, sep: str = " ") -> str:
    """
    Make italic text (HTML)

    :param content:
    :param sep:
    :return:
    """
    return html_decoration.italic(value=html_decoration.quote(_join(*content, sep=sep)))


def code(*content: Any, sep: str = " ") -> str:
    """
    Make mono-width text (Markdown)

    :param content:
    :param sep:
    :return:
    """
    return markdown_decoration.code(value=markdown_decoration.quote(_join(*content, sep=sep)))


def hcode(*content: Any, sep: str = " ") -> str:
    """
    Make mono-width text (HTML)

    :param content:
    :param sep:
    :return:
    """
    return html_decoration.code(value=html_decoration.quote(_join(*content, sep=sep)))


def pre(*content: Any, sep: str = "\n") -> str:
    """
    Make mono-width text block (Markdown)

    :param content:
    :param sep:
    :return:
    """
    return markdown_decoration.pre(value=markdown_decoration.quote(_join(*content, sep=sep)))


def hpre(*content: Any, sep: str = "\n") -> str:
    """
    Make mono-width text block (HTML)

    :param content:
    :param sep:
    :return:
    """
    return html_decoration.pre(value=html_decoration.quote(_join(*content, sep=sep)))


def underline(*content: Any, sep: str = " ") -> str:
    """
    Make underlined text (Markdown)

    :param content:
    :param sep:
    :return:
    """
    return markdown_decoration.underline(value=markdown_decoration.quote(_join(*content, sep=sep)))


def hunderline(*content: Any, sep: str = " ") -> str:
    """
    Make underlined text (HTML)

    :param content:
    :param sep:
    :return:
    """
    return html_decoration.underline(value=html_decoration.quote(_join(*content, sep=sep)))


def strikethrough(*content: Any, sep: str = " ") -> str:
    """
    Make strikethrough text (Markdown)

    :param content:
    :param sep:
    :return:
    """
    return markdown_decoration.strikethrough(
        value=markdown_decoration.quote(_join(*content, sep=sep)),
    )


def hstrikethrough(*content: Any, sep: str = " ") -> str:
    """
    Make strikethrough text (HTML)

    :param content:
    :param sep:
    :return:
    """
    return html_decoration.strikethrough(value=html_decoration.quote(_join(*content, sep=sep)))


def link(title: str, url: str) -> str:
    """
    Format URL (Markdown)

    :param title:
    :param url:
    :return:
    """
    return markdown_decoration.link(value=markdown_decoration.quote(title), link=url)


def hlink(title: str, url: str) -> str:
    """
    Format URL (HTML)

    :param title:
    :param url:
    :return:
    """
    return html_decoration.link(value=html_decoration.quote(title), link=url)


def blockquote(*content: Any, sep: str = "\n") -> str:
    """
    Make blockquote (Markdown)

    :param content:
    :param sep:
    :return:
    """
    return markdown_decoration.blockquote(
        value=markdown_decoration.quote(_join(*content, sep=sep)),
    )


def hblockquote(*content: Any, sep: str = "\n") -> str:
    """
    Make blockquote (HTML)

    :param content:
    :param sep:
    :return:
    """
    return html_decoration.blockquote(value=html_decoration.quote(_join(*content, sep=sep)))


def hide_link(url: str) -> str:
    """
    Hide URL (HTML only)
    Can be used for adding an image to a text message

    :param url:
    :return:
    """
    return f'<a href="{url}">&#8203;</a>'



from typing import TypeAlias

from .chat_member_administrator import ChatMemberAdministrator
from .chat_member_banned import ChatMemberBanned
from .chat_member_left import ChatMemberLeft
from .chat_member_member import ChatMemberMember
from .chat_member_owner import ChatMemberOwner
from .chat_member_restricted import ChatMemberRestricted

ResultChatMemberUnion: TypeAlias = (
    ChatMemberOwner
    | ChatMemberAdministrator
    | ChatMemberMember
    | ChatMemberRestricted
    | ChatMemberLeft
    | ChatMemberBanned
)



from __future__ import annotations

from typing import TYPE_CHECKING, Any

from .base import TelegramObject

if TYPE_CHECKING:
    from .location import Location


class BusinessLocation(TelegramObject):
    """
    Contains information about the location of a Telegram Business account.

    Source: https://core.telegram.org/bots/api#businesslocation
    """

    address: str
    """Address of the business"""
    location: Location | None = None
    """*Optional*. Location of the business"""

    if TYPE_CHECKING:
        # DO NOT EDIT MANUALLY!!!
        # This section was auto-generated via `butcher`

        def __init__(
            __pydantic__self__,
            *,
            address: str,
            location: Location | None = None,
            **__pydantic_kwargs: Any,
        ) -> None:
            # DO NOT EDIT MANUALLY!!!
            # This method was auto-generated via `butcher`
            # Is needed only for type checking and IDE support without any additional plugins

            super().__init__(address=address, location=location, **__pydantic_kwargs)



from __future__ import annotations

from ..types.gifts import Gifts
from .base import TelegramMethod


class GetAvailableGifts(TelegramMethod[Gifts]):
    """
    Returns the list of gifts that can be sent by the bot to users and channel chats. Requires no parameters. Returns a :class:`aiogram.types.gifts.Gifts` object.

    Source: https://core.telegram.org/bots/api#getavailablegifts
    """

    __returning__ = Gifts
    __api_method__ = "getAvailableGifts"



from abc import ABC, abstractmethod
from dataclasses import dataclass, field
from pathlib import Path
from typing import Any


class FilesPathWrapper(ABC):
    @abstractmethod
    def to_local(self, path: Path | str) -> Path | str:
        pass

    @abstractmethod
    def to_server(self, path: Path | str) -> Path | str:
        pass


class BareFilesPathWrapper(FilesPathWrapper):
    def to_local(self, path: Path | str) -> Path | str:
        return path

    def to_server(self, path: Path | str) -> Path | str:
        return path


class SimpleFilesPathWrapper(FilesPathWrapper):
    def __init__(self, server_path: Path, local_path: Path) -> None:
        self.server_path = server_path
        self.local_path = local_path

    @classmethod
    def _resolve(
        cls,
        base1: Path | str,
        base2: Path | str,
        value: Path | str,
    ) -> Path:
        relative = Path(value).relative_to(base1)
        return base2 / relative

    def to_local(self, path: Path | str) -> Path | str:
        return self._resolve(base1=self.server_path, base2=self.local_path, value=path)

    def to_server(self, path: Path | str) -> Path | str:
        return self._resolve(base1=self.local_path, base2=self.server_path, value=path)


@dataclass(frozen=True)
class TelegramAPIServer:
    """
    Base config for API Endpoints
    """

    base: str
    """Base URL"""
    file: str
    """Files URL"""
    is_local: bool = False
    """Mark this server is
    in `local mode <https://core.telegram.org/bots/api#using-a-local-bot-api-server>`_."""
    wrap_local_file: FilesPathWrapper = field(default=BareFilesPathWrapper())
    """Callback to wrap files path in local mode"""

    def api_url(self, token: str, method: str) -> str:
        """
        Generate URL for API methods

        :param token: Bot token
        :param method: API method name (case insensitive)
        :return: URL
        """
        return self.base.format(token=token, method=method)

    def file_url(self, token: str, path: str | Path) -> str:
        """
        Generate URL for downloading files

        :param token: Bot token
        :param path: file path
        :return: URL
        """
        return self.file.format(token=token, path=path)

    @classmethod
    def from_base(cls, base: str, **kwargs: Any) -> "TelegramAPIServer":
        """
        Use this method to auto-generate TelegramAPIServer instance from base URL

        :param base: Base URL
        :return: instance of :class:`TelegramAPIServer`
        """
        base = base.rstrip("/")
        return cls(
            base=f"{base}/bot{{token}}/{{method}}",
            file=f"{base}/file/bot{{token}}/{{path}}",
            **kwargs,
        )


PRODUCTION = TelegramAPIServer(
    base="https://api.telegram.org/bot{token}/{method}",
    file="https://api.telegram.org/file/bot{token}/{path}",
)
TEST = TelegramAPIServer(
    base="https://api.telegram.org/bot{token}/test/{method}",
    file="https://api.telegram.org/file/bot{token}/test/{path}",
)



from __future__ import annotations

from typing import TYPE_CHECKING, Any

from pydantic import Field

from .base import TelegramObject

if TYPE_CHECKING:
    from ..methods import AnswerCallbackQuery
    from .maybe_inaccessible_message_union import MaybeInaccessibleMessageUnion
    from .user import User


class CallbackQuery(TelegramObject):
    """
    This object represents an incoming callback query from a callback button in an `inline keyboard <https://core.telegram.org/bots/features#inline-keyboards>`_. If the button that originated the query was attached to a message sent by the bot, the field *message* will be present. If the button was attached to a message sent via the bot (in `inline mode <https://core.telegram.org/bots/api#inline-mode>`_), the field *inline_message_id* will be present. Exactly one of the fields *data* or *game_short_name* will be present.

     **NOTE:** After the user presses a callback button, Telegram clients will display a progress bar until you call :class:`aiogram.methods.answer_callback_query.AnswerCallbackQuery`. It is, therefore, necessary to react by calling :class:`aiogram.methods.answer_callback_query.AnswerCallbackQuery` even if no notification to the user is needed (e.g., without specifying any of the optional parameters).

    Source: https://core.telegram.org/bots/api#callbackquery
    """

    id: str
    """Unique identifier for this query"""
    from_user: User = Field(..., alias="from")
    """Sender"""
    chat_instance: str
    """Global identifier, uniquely corresponding to the chat to which the message with the callback button was sent. Useful for high scores in :class:`aiogram.methods.games.Games`."""
    message: MaybeInaccessibleMessageUnion | None = None
    """*Optional*. Message sent by the bot with the callback button that originated the query"""
    inline_message_id: str | None = None
    """*Optional*. Identifier of the message sent via the bot in inline mode, that originated the query."""
    data: str | None = None
    """*Optional*. Data associated with the callback button. Be aware that the message originated the query can contain no callback buttons with this data."""
    game_short_name: str | None = None
    """*Optional*. Short name of a `Game <https://core.telegram.org/bots/api#games>`_ to be returned, serves as the unique identifier for the game"""

    if TYPE_CHECKING:
        # DO NOT EDIT MANUALLY!!!
        # This section was auto-generated via `butcher`

        def __init__(
            __pydantic__self__,
            *,
            id: str,
            from_user: User,
            chat_instance: str,
            message: MaybeInaccessibleMessageUnion | None = None,
            inline_message_id: str | None = None,
            data: str | None = None,
            game_short_name: str | None = None,
            **__pydantic_kwargs: Any,
        ) -> None:
            # DO NOT EDIT MANUALLY!!!
            # This method was auto-generated via `butcher`
            # Is needed only for type checking and IDE support without any additional plugins

            super().__init__(
                id=id,
                from_user=from_user,
                chat_instance=chat_instance,
                message=message,
                inline_message_id=inline_message_id,
                data=data,
                game_short_name=game_short_name,
                **__pydantic_kwargs,
            )

    def answer(
        self,
        text: str | None = None,
        show_alert: bool | None = None,
        url: str | None = None,
        cache_time: int | None = None,
        **kwargs: Any,
    ) -> AnswerCallbackQuery:
        """
        Shortcut for method :class:`aiogram.methods.answer_callback_query.AnswerCallbackQuery`
        will automatically fill method attributes:

        - :code:`callback_query_id`

        Use this method to send answers to callback queries sent from `inline keyboards <https://core.telegram.org/bots/features#inline-keyboards>`_. The answer will be displayed to the user as a notification at the top of the chat screen or as an alert. On success, :code:`True` is returned.

         Alternatively, the user can be redirected to the specified Game URL. For this option to work, you must first create a game for your bot via `@BotFather <https://t.me/botfather>`_ and accept the terms. Otherwise, you may use links like :code:`t.me/your_bot?start=XXXX` that open your bot with a parameter.

        Source: https://core.telegram.org/bots/api#answercallbackquery

        :param text: Text of the notification. If not specified, nothing will be shown to the user, 0-200 characters
        :param show_alert: If :code:`True`, an alert will be shown by the client instead of a notification at the top of the chat screen. Defaults to *false*.
        :param url: URL that will be opened by the user's client. If you have created a :class:`aiogram.types.game.Game` and accepted the conditions via `@BotFather <https://t.me/botfather>`_, specify the URL that opens your game - note that this will only work if the query comes from a `https://core.telegram.org/bots/api#inlinekeyboardbutton <https://core.telegram.org/bots/api#inlinekeyboardbutton>`_ *callback_game* button.
        :param cache_time: The maximum amount of time in seconds that the result of the callback query may be cached client-side. Telegram apps will support caching starting in version 3.14. Defaults to 0.
        :return: instance of method :class:`aiogram.methods.answer_callback_query.AnswerCallbackQuery`
        """
        # DO NOT EDIT MANUALLY!!!
        # This method was auto-generated via `butcher`

        from aiogram.methods import AnswerCallbackQuery

        return AnswerCallbackQuery(
            callback_query_id=self.id,
            text=text,
            show_alert=show_alert,
            url=url,
            cache_time=cache_time,
            **kwargs,
        ).as_(self._bot)



from enum import Enum


class MaskPositionPoint(str, Enum):
    """
    The part of the face relative to which the mask should be placed.

    Source: https://core.telegram.org/bots/api#maskposition
    """

    FOREHEAD = "forehead"
    EYES = "eyes"
    MOUTH = "mouth"
    CHIN = "chin"



from __future__ import annotations

from typing import TYPE_CHECKING, Any, Literal

from ..enums import MenuButtonType
from .menu_button import MenuButton

if TYPE_CHECKING:
    from .web_app_info import WebAppInfo


class MenuButtonWebApp(MenuButton):
    """
    Represents a menu button, which launches a `Web App <https://core.telegram.org/bots/webapps>`_.

    Source: https://core.telegram.org/bots/api#menubuttonwebapp
    """

    type: Literal[MenuButtonType.WEB_APP] = MenuButtonType.WEB_APP
    """Type of the button, must be *web_app*"""
    text: str
    """Text on the button"""
    web_app: WebAppInfo
    """Description of the Web App that will be launched when the user presses the button. The Web App will be able to send an arbitrary message on behalf of the user using the method :class:`aiogram.methods.answer_web_app_query.AnswerWebAppQuery`. Alternatively, a :code:`t.me` link to a Web App of the bot can be specified in the object instead of the Web App's URL, in which case the Web App will be opened as if the user pressed the link."""

    if TYPE_CHECKING:
        # DO NOT EDIT MANUALLY!!!
        # This section was auto-generated via `butcher`

        def __init__(
            __pydantic__self__,
            *,
            type: Literal[MenuButtonType.WEB_APP] = MenuButtonType.WEB_APP,
            text: str,
            web_app: WebAppInfo,
            **__pydantic_kwargs: Any,
        ) -> None:
            # DO NOT EDIT MANUALLY!!!
            # This method was auto-generated via `butcher`
            # Is needed only for type checking and IDE support without any additional plugins

            super().__init__(type=type, text=text, web_app=web_app, **__pydantic_kwargs)



from enum import Enum


class InputProfilePhotoType(str, Enum):
    """
    This object represents input profile photo type

    Source: https://core.telegram.org/bots/api#inputprofilephoto
    """

    STATIC = "static"
    ANIMATED = "animated"



from __future__ import annotations

from typing import TYPE_CHECKING, Any

from .base import TelegramObject
from .custom import DateTime

if TYPE_CHECKING:
    from .transaction_partner_union import TransactionPartnerUnion


class StarTransaction(TelegramObject):
    """
    Describes a Telegram Star transaction. Note that if the buyer initiates a chargeback with the payment provider from whom they acquired Stars (e.g., Apple, Google) following this transaction, the refunded Stars will be deducted from the bot's balance. This is outside of Telegram's control.

    Source: https://core.telegram.org/bots/api#startransaction
    """

    id: str
    """Unique identifier of the transaction. Coincides with the identifier of the original transaction for refund transactions. Coincides with *SuccessfulPayment.telegram_payment_charge_id* for successful incoming payments from users."""
    amount: int
    """Integer amount of Telegram Stars transferred by the transaction"""
    date: DateTime
    """Date the transaction was created in Unix time"""
    nanostar_amount: int | None = None
    """*Optional*. The number of 1/1000000000 shares of Telegram Stars transferred by the transaction; from 0 to 999999999"""
    source: TransactionPartnerUnion | None = None
    """*Optional*. Source of an incoming transaction (e.g., a user purchasing goods or services, Fragment refunding a failed withdrawal). Only for incoming transactions"""
    receiver: TransactionPartnerUnion | None = None
    """*Optional*. Receiver of an outgoing transaction (e.g., a user for a purchase refund, Fragment for a withdrawal). Only for outgoing transactions"""

    if TYPE_CHECKING:
        # DO NOT EDIT MANUALLY!!!
        # This section was auto-generated via `butcher`

        def __init__(
            __pydantic__self__,
            *,
            id: str,
            amount: int,
            date: DateTime,
            nanostar_amount: int | None = None,
            source: TransactionPartnerUnion | None = None,
            receiver: TransactionPartnerUnion | None = None,
            **__pydantic_kwargs: Any,
        ) -> None:
            # DO NOT EDIT MANUALLY!!!
            # This method was auto-generated via `butcher`
            # Is needed only for type checking and IDE support without any additional plugins

            super().__init__(
                id=id,
                amount=amount,
                date=date,
                nanostar_amount=nanostar_amount,
                source=source,
                receiver=receiver,
                **__pydantic_kwargs,
            )



from __future__ import annotations

from typing import TYPE_CHECKING, Any

from .base import TelegramObject

if TYPE_CHECKING:
    from .message_entity import MessageEntity


class TextQuote(TelegramObject):
    """
    This object contains information about the quoted part of a message that is replied to by the given message.

    Source: https://core.telegram.org/bots/api#textquote
    """

    text: str
    """Text of the quoted part of a message that is replied to by the given message"""
    position: int
    """Approximate quote position in the original message in UTF-16 code units as specified by the sender"""
    entities: list[MessageEntity] | None = None
    """*Optional*. Special entities that appear in the quote. Currently, only *bold*, *italic*, *underline*, *strikethrough*, *spoiler*, *custom_emoji*, and *date_time* entities are kept in quotes."""
    is_manual: bool | None = None
    """*Optional*. :code:`True`, if the quote was chosen manually by the message sender. Otherwise, the quote was added automatically by the server."""

    if TYPE_CHECKING:
        # DO NOT EDIT MANUALLY!!!
        # This section was auto-generated via `butcher`

        def __init__(
            __pydantic__self__,
            *,
            text: str,
            position: int,
            entities: list[MessageEntity] | None = None,
            is_manual: bool | None = None,
            **__pydantic_kwargs: Any,
        ) -> None:
            # DO NOT EDIT MANUALLY!!!
            # This method was auto-generated via `butcher`
            # Is needed only for type checking and IDE support without any additional plugins

            super().__init__(
                text=text,
                position=position,
                entities=entities,
                is_manual=is_manual,
                **__pydantic_kwargs,
            )



from __future__ import annotations

import html
import re
from abc import ABC, abstractmethod
from datetime import date, datetime, time
from typing import TYPE_CHECKING, cast

from aiogram.enums import MessageEntityType
from aiogram.utils.link import create_tg_link

if TYPE_CHECKING:
    from collections.abc import Generator
    from re import Pattern

    from aiogram.types import MessageEntity

__all__ = (
    "HtmlDecoration",
    "MarkdownDecoration",
    "TextDecoration",
    "add_surrogates",
    "html_decoration",
    "markdown_decoration",
    "remove_surrogates",
)


def add_surrogates(text: str) -> bytes:
    return text.encode("utf-16-le")


def remove_surrogates(text: bytes) -> str:
    return text.decode("utf-16-le")


class TextDecoration(ABC):
    def apply_entity(self, entity: MessageEntity, text: str) -> str:
        """
        Apply single entity to text

        :param entity:
        :param text:
        :return:
        """
        if entity.type in {
            MessageEntityType.BOT_COMMAND,
            MessageEntityType.URL,
            MessageEntityType.MENTION,
            MessageEntityType.PHONE_NUMBER,
            MessageEntityType.HASHTAG,
            MessageEntityType.CASHTAG,
            MessageEntityType.EMAIL,
        }:
            # These entities should not be changed
            return text
        if entity.type in {
            MessageEntityType.BOLD,
            MessageEntityType.ITALIC,
            MessageEntityType.CODE,
            MessageEntityType.UNDERLINE,
            MessageEntityType.STRIKETHROUGH,
            MessageEntityType.SPOILER,
            MessageEntityType.BLOCKQUOTE,
            MessageEntityType.EXPANDABLE_BLOCKQUOTE,
        }:
            return cast(str, getattr(self, entity.type)(value=text))
        if entity.type == MessageEntityType.PRE:
            return (
                self.pre_language(value=text, language=entity.language)
                if entity.language
                else self.pre(value=text)
            )
        if entity.type == MessageEntityType.TEXT_MENTION:
            from aiogram.types import User

            user = cast(User, entity.user)
            return self.link(value=text, link=f"tg://user?id={user.id}")
        if entity.type == MessageEntityType.TEXT_LINK:
            return self.link(value=text, link=cast(str, entity.url))
        if entity.type == MessageEntityType.CUSTOM_EMOJI:
            return self.custom_emoji(value=text, custom_emoji_id=cast(str, entity.custom_emoji_id))
        if entity.type == MessageEntityType.DATE_TIME:
            return self.date_time(
                value=text,
                unix_time=cast(int, entity.unix_time),
                date_time_format=entity.date_time_format,
            )

        # This case is not possible because of `if` above, but if any new entity is added to
        # API it will be here too
        return self.quote(text)

    def unparse(self, text: str, entities: list[MessageEntity] | None = None) -> str:
        """
        Unparse message entities

        :param text: raw text
        :param entities: Array of MessageEntities
        :return:
        """
        return "".join(
            self._unparse_entities(
                add_surrogates(text),
                sorted(entities, key=lambda item: item.offset) if entities else [],
            ),
        )

    def _unparse_entities(
        self,
        text: bytes,
        entities: list[MessageEntity],
        offset: int | None = None,
        length: int | None = None,
    ) -> Generator[str, None, None]:
        if offset is None:
            offset = 0
        length = length or len(text)

        for index, entity in enumerate(entities):
            if entity.offset * 2 < offset:
                continue
            if entity.offset * 2 > offset:
                yield self.quote(remove_surrogates(text[offset : entity.offset * 2]))
            start = entity.offset * 2
            offset = entity.offset * 2 + entity.length * 2

            sub_entities = list(
                filter(lambda e: e.offset * 2 < (offset or 0), entities[index + 1 :]),
            )
            yield self.apply_entity(
                entity,
                "".join(self._unparse_entities(text, sub_entities, offset=start, length=offset)),
            )

        if offset < length:
            yield self.quote(remove_surrogates(text[offset:length]))

    @abstractmethod
    def link(self, value: str, link: str) -> str:
        pass

    @abstractmethod
    def bold(self, value: str) -> str:
        pass

    @abstractmethod
    def italic(self, value: str) -> str:
        pass

    @abstractmethod
    def code(self, value: str) -> str:
        pass

    @abstractmethod
    def pre(self, value: str) -> str:
        pass

    @abstractmethod
    def pre_language(self, value: str, language: str) -> str:
        pass

    @abstractmethod
    def underline(self, value: str) -> str:
        pass

    @abstractmethod
    def strikethrough(self, value: str) -> str:
        pass

    @abstractmethod
    def spoiler(self, value: str) -> str:
        pass

    @abstractmethod
    def quote(self, value: str) -> str:
        pass

    @abstractmethod
    def custom_emoji(self, value: str, custom_emoji_id: str) -> str:
        pass

    @abstractmethod
    def blockquote(self, value: str) -> str:
        pass

    @abstractmethod
    def expandable_blockquote(self, value: str) -> str:
        pass

    @abstractmethod
    def date_time(
        self,
        value: str,
        unix_time: int | datetime,
        date_time_format: str | None = None,
    ) -> str:
        pass


class HtmlDecoration(TextDecoration):
    BOLD_TAG = "b"
    ITALIC_TAG = "i"
    UNDERLINE_TAG = "u"
    STRIKETHROUGH_TAG = "s"
    CODE_TAG = "code"
    PRE_TAG = "pre"
    LINK_TAG = "a"
    SPOILER_TAG = "tg-spoiler"
    EMOJI_TAG = "tg-emoji"
    DATE_TIME_TAG = "tg-time"
    BLOCKQUOTE_TAG = "blockquote"

    def _tag(
        self,
        tag: str,
        content: str,
        *,
        attrs: dict[str, str] | None = None,
        flags: list[str] | None = None,
    ) -> str:
        prepared_attrs: list[str] = []
        if attrs:
            prepared_attrs.extend(f'{k}="{v}"' for k, v in attrs.items())
        if flags:
            prepared_attrs.extend(f"{flag}" for flag in flags)

        attrs_str = " ".join(prepared_attrs)
        if attrs_str:
            attrs_str = " " + attrs_str

        return f"<{tag}{attrs_str}>{content}</{tag}>"

    def link(self, value: str, link: str) -> str:
        return self._tag(self.LINK_TAG, value, attrs={"href": link})

    def bold(self, value: str) -> str:
        return self._tag(self.BOLD_TAG, value)

    def italic(self, value: str) -> str:
        return self._tag(self.ITALIC_TAG, value)

    def code(self, value: str) -> str:
        return self._tag(self.CODE_TAG, value)

    def pre(self, value: str) -> str:
        return self._tag(self.PRE_TAG, value)

    def pre_language(self, value: str, language: str) -> str:
        return self._tag(
            self.PRE_TAG,
            self._tag(self.CODE_TAG, value, attrs={"language": f"language-{language}"}),
        )

    def underline(self, value: str) -> str:
        return self._tag(self.UNDERLINE_TAG, value)

    def strikethrough(self, value: str) -> str:
        return self._tag(self.STRIKETHROUGH_TAG, value)

    def spoiler(self, value: str) -> str:
        return self._tag(self.SPOILER_TAG, value)

    def quote(self, value: str) -> str:
        return html.escape(value, quote=False)

    def custom_emoji(self, value: str, custom_emoji_id: str) -> str:
        return self._tag(self.EMOJI_TAG, value, attrs={"emoji-id": custom_emoji_id})

    def blockquote(self, value: str) -> str:
        return self._tag(self.BLOCKQUOTE_TAG, value)

    def expandable_blockquote(self, value: str) -> str:
        return self._tag(self.BLOCKQUOTE_TAG, value, flags=["expandable"])

    def date_time(
        self,
        value: str,
        unix_time: int | datetime,
        date_time_format: str | None = None,
    ) -> str:
        if isinstance(unix_time, datetime):
            unix_time = int(unix_time.timestamp())

        args = {"unix": str(unix_time)}
        if date_time_format:
            args["format"] = date_time_format

        return self._tag(self.DATE_TIME_TAG, value, attrs=args)


class MarkdownDecoration(TextDecoration):
    MARKDOWN_QUOTE_PATTERN: Pattern[str] = re.compile(r"([_*\[\]()~`>#+\-=|{}.!\\])")

    def link(self, value: str, link: str) -> str:
        return f"[{value}]({link})"

    def bold(self, value: str) -> str:
        return f"*{value}*"

    def italic(self, value: str) -> str:
        return f"_\r{value}_\r"

    def code(self, value: str) -> str:
        return f"`{value}`"

    def pre(self, value: str) -> str:
        return f"```\n{value}\n```"

    def pre_language(self, value: str, language: str) -> str:
        return f"```{language}\n{value}\n```"

    def underline(self, value: str) -> str:
        return f"__\r{value}__\r"

    def strikethrough(self, value: str) -> str:
        return f"~{value}~"

    def spoiler(self, value: str) -> str:
        return f"||{value}||"

    def quote(self, value: str) -> str:
        return re.sub(pattern=self.MARKDOWN_QUOTE_PATTERN, repl=r"\\\1", string=value)

    def custom_emoji(self, value: str, custom_emoji_id: str) -> str:
        link = create_tg_link("emoji", emoji_id=custom_emoji_id)
        return f"!{self.link(value=value, link=link)}"

    def blockquote(self, value: str) -> str:
        return "\n".join(f">{line}" for line in value.splitlines())

    def expandable_blockquote(self, value: str) -> str:
        return "\n".join(f">{line}" for line in value.splitlines()) + "||"

    def date_time(
        self,
        value: str,
        unix_time: int | datetime,
        date_time_format: str | None = None,
    ) -> str:
        if isinstance(unix_time, datetime):
            unix_time = int(unix_time.timestamp())

        link_params = {"unix": str(unix_time)}
        if date_time_format:
            link_params["format"] = date_time_format
        link = create_tg_link("time", **link_params)

        return f"!{self.link(value, link=link)}"


html_decoration = HtmlDecoration()
markdown_decoration = MarkdownDecoration()



from .context import get_i18n, gettext, lazy_gettext, lazy_ngettext, ngettext
from .core import I18n
from .middleware import (
    ConstI18nMiddleware,
    FSMI18nMiddleware,
    I18nMiddleware,
    SimpleI18nMiddleware,
)

__all__ = (
    "ConstI18nMiddleware",
    "FSMI18nMiddleware",
    "I18n",
    "I18nMiddleware",
    "SimpleI18nMiddleware",
    "get_i18n",
    "gettext",
    "lazy_gettext",
    "lazy_ngettext",
    "ngettext",
)



from enum import Enum


class TransactionPartnerType(str, Enum):
    """
    This object represents a type of transaction partner.

    Source: https://core.telegram.org/bots/api#transactionpartner
    """

    FRAGMENT = "fragment"
    OTHER = "other"
    USER = "user"
    TELEGRAM_ADS = "telegram_ads"
    TELEGRAM_API = "telegram_api"
    AFFILIATE_PROGRAM = "affiliate_program"
    CHAT = "chat"



from __future__ import annotations

from typing import TYPE_CHECKING, Any, Literal

from ..enums import PaidMediaType
from .paid_media import PaidMedia


class PaidMediaPreview(PaidMedia):
    """
    The paid media isn't available before the payment.

    Source: https://core.telegram.org/bots/api#paidmediapreview
    """

    type: Literal[PaidMediaType.PREVIEW] = PaidMediaType.PREVIEW
    """Type of the paid media, always 'preview'"""
    width: int | None = None
    """*Optional*. Media width as defined by the sender"""
    height: int | None = None
    """*Optional*. Media height as defined by the sender"""
    duration: int | None = None
    """*Optional*. Duration of the media in seconds as defined by the sender"""

    if TYPE_CHECKING:
        # DO NOT EDIT MANUALLY!!!
        # This section was auto-generated via `butcher`

        def __init__(
            __pydantic__self__,
            *,
            type: Literal[PaidMediaType.PREVIEW] = PaidMediaType.PREVIEW,
            width: int | None = None,
            height: int | None = None,
            duration: int | None = None,
            **__pydantic_kwargs: Any,
        ) -> None:
            # DO NOT EDIT MANUALLY!!!
            # This method was auto-generated via `butcher`
            # Is needed only for type checking and IDE support without any additional plugins

            super().__init__(
                type=type, width=width, height=height, duration=duration, **__pydantic_kwargs
            )



from __future__ import annotations

from typing import TYPE_CHECKING, Any

from .base import TelegramMethod


class SetMyDescription(TelegramMethod[bool]):
    """
    Use this method to change the bot's description, which is shown in the chat with the bot if the chat is empty. Returns :code:`True` on success.

    Source: https://core.telegram.org/bots/api#setmydescription
    """

    __returning__ = bool
    __api_method__ = "setMyDescription"

    description: str | None = None
    """New bot description; 0-512 characters. Pass an empty string to remove the dedicated description for the given language."""
    language_code: str | None = None
    """A two-letter ISO 639-1 language code. If empty, the description will be applied to all users for whose language there is no dedicated description."""

    if TYPE_CHECKING:
        # DO NOT EDIT MANUALLY!!!
        # This section was auto-generated via `butcher`

        def __init__(
            __pydantic__self__,
            *,
            description: str | None = None,
            language_code: str | None = None,
            **__pydantic_kwargs: Any,
        ) -> None:
            # DO NOT EDIT MANUALLY!!!
            # This method was auto-generated via `butcher`
            # Is needed only for type checking and IDE support without any additional plugins

            super().__init__(
                description=description, language_code=language_code, **__pydantic_kwargs
            )



from abc import ABC
from typing import TYPE_CHECKING, Any

from aiogram.filters import Filter

if TYPE_CHECKING:
    from aiogram.dispatcher.event.handler import CallbackType, FilterObject


class _LogicFilter(Filter, ABC):
    pass


class _InvertFilter(_LogicFilter):
    __slots__ = ("target",)

    def __init__(self, target: "FilterObject") -> None:
        self.target = target

    async def __call__(self, *args: Any, **kwargs: Any) -> bool | dict[str, Any]:
        return not bool(await self.target.call(*args, **kwargs))


class _AndFilter(_LogicFilter):
    __slots__ = ("targets",)

    def __init__(self, *targets: "FilterObject") -> None:
        self.targets = targets

    async def __call__(self, *args: Any, **kwargs: Any) -> bool | dict[str, Any]:
        final_result = {}

        for target in self.targets:
            result = await target.call(*args, **kwargs)
            if not result:
                return False
            if isinstance(result, dict):
                final_result.update(result)

        if final_result:
            return final_result
        return True


class _OrFilter(_LogicFilter):
    __slots__ = ("targets",)

    def __init__(self, *targets: "FilterObject") -> None:
        self.targets = targets

    async def __call__(self, *args: Any, **kwargs: Any) -> bool | dict[str, Any]:
        for target in self.targets:
            result = await target.call(*args, **kwargs)
            if not result:
                continue
            if isinstance(result, dict):
                return result
            return bool(result)
        return False


def and_f(*targets: "CallbackType") -> _AndFilter:
    from aiogram.dispatcher.event.handler import FilterObject

    return _AndFilter(*(FilterObject(target) for target in targets))


def or_f(*targets: "CallbackType") -> _OrFilter:
    from aiogram.dispatcher.event.handler import FilterObject

    return _OrFilter(*(FilterObject(target) for target in targets))


def invert_f(target: "CallbackType") -> _InvertFilter:
    from aiogram.dispatcher.event.handler import FilterObject

    return _InvertFilter(FilterObject(target))



from typing import Any

from aiogram.utils.i18n.core import I18n
from aiogram.utils.i18n.lazy_proxy import LazyProxy


def get_i18n() -> I18n:
    i18n = I18n.get_current(no_error=True)
    if i18n is None:
        msg = "I18n context is not set"
        raise LookupError(msg)
    return i18n


def gettext(*args: Any, **kwargs: Any) -> str:
    return get_i18n().gettext(*args, **kwargs)


def lazy_gettext(*args: Any, **kwargs: Any) -> LazyProxy:
    return LazyProxy(gettext, *args, **kwargs, enable_cache=False)


ngettext = gettext
lazy_ngettext = lazy_gettext



from __future__ import annotations

from typing import TYPE_CHECKING, Any, Literal

from ..enums import PassportElementErrorType
from .passport_element_error import PassportElementError


class PassportElementErrorFiles(PassportElementError):
    """
    Represents an issue with a list of scans. The error is considered resolved when the list of files containing the scans changes.

    Source: https://core.telegram.org/bots/api#passportelementerrorfiles
    """

    source: Literal[PassportElementErrorType.FILES] = PassportElementErrorType.FILES
    """Error source, must be *files*"""
    type: str
    """The section of the user's Telegram Passport which has the issue, one of 'utility_bill', 'bank_statement', 'rental_agreement', 'passport_registration', 'temporary_registration'"""
    file_hashes: list[str]
    """List of base64-encoded file hashes"""
    message: str
    """Error message"""

    if TYPE_CHECKING:
        # DO NOT EDIT MANUALLY!!!
        # This section was auto-generated via `butcher`

        def __init__(
            __pydantic__self__,
            *,
            source: Literal[PassportElementErrorType.FILES] = PassportElementErrorType.FILES,
            type: str,
            file_hashes: list[str],
            message: str,
            **__pydantic_kwargs: Any,
        ) -> None:
            # DO NOT EDIT MANUALLY!!!
            # This method was auto-generated via `butcher`
            # Is needed only for type checking and IDE support without any additional plugins

            super().__init__(
                source=source,
                type=type,
                file_hashes=file_hashes,
                message=message,
                **__pydantic_kwargs,
            )



from typing import Any

from aiogram.methods import TelegramMethod
from aiogram.methods.base import TelegramType
from aiogram.utils.link import docs_url


class AiogramError(Exception):
    """
    Base exception for all aiogram errors.
    """


class DetailedAiogramError(AiogramError):
    """
    Base exception for all aiogram errors with detailed message.
    """

    url: str | None = None

    def __init__(self, message: str) -> None:
        self.message = message

    def __str__(self) -> str:
        message = self.message
        if self.url:
            message += f"\n(background on this error at: {self.url})"
        return message

    def __repr__(self) -> str:
        return f"{type(self).__name__}('{self}')"


class CallbackAnswerException(AiogramError):
    """
    Exception for callback answer.
    """


class SceneException(AiogramError):
    """
    Exception for scenes.
    """


class UnsupportedKeywordArgument(DetailedAiogramError):
    """
    Exception raised when a keyword argument is passed as filter.
    """

    url = docs_url("migration_2_to_3.html", fragment_="filtering-events")


class TelegramAPIError(DetailedAiogramError):
    """
    Base exception for all Telegram API errors.
    """

    label: str = "Telegram server says"

    def __init__(
        self,
        method: TelegramMethod[TelegramType],
        message: str,
    ) -> None:
        super().__init__(message=message)
        self.method = method

    def __str__(self) -> str:
        original_message = super().__str__()
        return f"{self.label} - {original_message}"


class TelegramNetworkError(TelegramAPIError):
    """
    Base exception for all Telegram network errors.
    """

    label = "HTTP Client says"


class TelegramRetryAfter(TelegramAPIError):
    """
    Exception raised when flood control exceeds.
    """

    url = "https://core.telegram.org/bots/faq#my-bot-is-hitting-limits-how-do-i-avoid-this"

    def __init__(
        self,
        method: TelegramMethod[TelegramType],
        message: str,
        retry_after: int,
    ) -> None:
        description = f"Flood control exceeded on method {type(method).__name__!r}"
        if chat_id := getattr(method, "chat_id", None):
            description += f" in chat {chat_id}"
        description += f". Retry in {retry_after} seconds."
        description += f"\nOriginal description: {message}"

        super().__init__(method=method, message=description)
        self.retry_after = retry_after


class TelegramMigrateToChat(TelegramAPIError):
    """
    Exception raised when chat has been migrated to a supergroup.
    """

    url = "https://core.telegram.org/bots/api#responseparameters"

    def __init__(
        self,
        method: TelegramMethod[TelegramType],
        message: str,
        migrate_to_chat_id: int,
    ) -> None:
        description = f"The group has been migrated to a supergroup with id {migrate_to_chat_id}"
        if chat_id := getattr(method, "chat_id", None):
            description += f" from {chat_id}"
        description += f"\nOriginal description: {message}"
        super().__init__(method=method, message=description)
        self.migrate_to_chat_id = migrate_to_chat_id


class TelegramBadRequest(TelegramAPIError):
    """
    Exception raised when request is malformed.
    """


class TelegramNotFound(TelegramAPIError):
    """
    Exception raised when chat, message, user, etc. not found.
    """


class TelegramConflictError(TelegramAPIError):
    """
    Exception raised when bot token is already used by another application in polling mode.
    """


class TelegramUnauthorizedError(TelegramAPIError):
    """
    Exception raised when bot token is invalid.
    """


class TelegramForbiddenError(TelegramAPIError):
    """
    Exception raised when bot is kicked from chat or etc.
    """


class TelegramServerError(TelegramAPIError):
    """
    Exception raised when Telegram server returns 5xx error.
    """


class RestartingTelegram(TelegramServerError):
    """
    Exception raised when Telegram server is restarting.

    It seems like this error is not used by Telegram anymore,
    but it's still here for backward compatibility.

    Currently, you should expect that Telegram can raise RetryAfter (with timeout 5 seconds)
     error instead of this one.
    """


class TelegramEntityTooLarge(TelegramNetworkError):
    """
    Exception raised when you are trying to send a file that is too large.
    """

    url = "https://core.telegram.org/bots/api#sending-files"


class ClientDecodeError(AiogramError):
    """
    Exception raised when client can't decode response. (Malformed response, etc.)
    """

    def __init__(self, message: str, original: Exception, data: Any) -> None:
        self.message = message
        self.original = original
        self.data = data

    def __str__(self) -> str:
        original_type = type(self.original)
        return (
            f"{self.message}\n"
            f"Caused from error: "
            f"{original_type.__module__}.{original_type.__name__}: {self.original}\n"
            f"Content: {self.data}"
        )


class DataNotDictLikeError(DetailedAiogramError):
    """
    Exception raised when data is not dict-like.
    """



from __future__ import annotations

from typing import TYPE_CHECKING, Any

from ..types import ChatIdUnion, ChatInviteLink, DateTimeUnion
from .base import TelegramMethod


class CreateChatSubscriptionInviteLink(TelegramMethod[ChatInviteLink]):
    """
    Use this method to create a `subscription invite link <https://telegram.org/blog/superchannels-star-reactions-subscriptions#star-subscriptions>`_ for a channel chat. The bot must have the *can_invite_users* administrator rights. The link can be edited using the method :class:`aiogram.methods.edit_chat_subscription_invite_link.EditChatSubscriptionInviteLink` or revoked using the method :class:`aiogram.methods.revoke_chat_invite_link.RevokeChatInviteLink`. Returns the new invite link as a :class:`aiogram.types.chat_invite_link.ChatInviteLink` object.

    Source: https://core.telegram.org/bots/api#createchatsubscriptioninvitelink
    """

    __returning__ = ChatInviteLink
    __api_method__ = "createChatSubscriptionInviteLink"

    chat_id: ChatIdUnion
    """Unique identifier for the target channel chat or username of the target channel in the format :code:`@username`"""
    subscription_period: DateTimeUnion
    """The number of seconds the subscription will be active for before the next payment. Currently, it must always be 2592000 (30 days)."""
    subscription_price: int
    """The amount of Telegram Stars a user must pay initially and after each subsequent subscription period to be a member of the chat; 1-10000"""
    name: str | None = None
    """Invite link name; 0-32 characters"""

    if TYPE_CHECKING:
        # DO NOT EDIT MANUALLY!!!
        # This section was auto-generated via `butcher`

        def __init__(
            __pydantic__self__,
            *,
            chat_id: ChatIdUnion,
            subscription_period: DateTimeUnion,
            subscription_price: int,
            name: str | None = None,
            **__pydantic_kwargs: Any,
        ) -> None:
            # DO NOT EDIT MANUALLY!!!
            # This method was auto-generated via `butcher`
            # Is needed only for type checking and IDE support without any additional plugins

            super().__init__(
                chat_id=chat_id,
                subscription_period=subscription_period,
                subscription_price=subscription_price,
                name=name,
                **__pydantic_kwargs,
            )



from enum import Enum


class PassportElementErrorType(str, Enum):
    """
    This object represents a passport element error type.

    Source: https://core.telegram.org/bots/api#passportelementerror
    """

    DATA = "data"
    FRONT_SIDE = "front_side"
    REVERSE_SIDE = "reverse_side"
    SELFIE = "selfie"
    FILE = "file"
    FILES = "files"
    TRANSLATION_FILE = "translation_file"
    TRANSLATION_FILES = "translation_files"
    UNSPECIFIED = "unspecified"



from __future__ import annotations

from typing import TypeAlias

from .paid_media_live_photo import PaidMediaLivePhoto
from .paid_media_photo import PaidMediaPhoto
from .paid_media_preview import PaidMediaPreview
from .paid_media_video import PaidMediaVideo

PaidMediaUnion: TypeAlias = PaidMediaLivePhoto | PaidMediaPhoto | PaidMediaPreview | PaidMediaVideo



from __future__ import annotations

from collections.abc import Callable
from typing import TYPE_CHECKING, Any

from aiogram.dispatcher.middlewares.manager import MiddlewareManager
from aiogram.exceptions import UnsupportedKeywordArgument
from aiogram.filters.base import Filter

from .bases import UNHANDLED, MiddlewareType, SkipHandler
from .handler import CallbackType, FilterObject, HandlerObject

if TYPE_CHECKING:
    from aiogram.dispatcher.router import Router
    from aiogram.types import TelegramObject


class TelegramEventObserver:
    """
    Event observer for Telegram events

    Here you can register handler with filter.
    This observer will stop event propagation when first handler is pass.
    """

    def __init__(self, router: Router, event_name: str) -> None:
        self.router: Router = router
        self.event_name: str = event_name

        self.handlers: list[HandlerObject] = []

        self.middleware = MiddlewareManager()
        self.outer_middleware = MiddlewareManager()

        # Re-used filters check method from already implemented handler object
        # with dummy callback which never will be used
        self._handler = HandlerObject(callback=lambda: True, filters=[])

    def filter(self, *filters: CallbackType) -> None:
        """
        Register filter for all handlers of this event observer

        :param filters: positional filters
        """
        if self._handler.filters is None:
            self._handler.filters = []
        self._handler.filters.extend([FilterObject(filter_) for filter_ in filters])

    def _resolve_middlewares(self) -> list[MiddlewareType[TelegramObject]]:
        middlewares: list[MiddlewareType[TelegramObject]] = []
        for router in reversed(tuple(self.router.chain_head)):
            observer = router.observers.get(self.event_name)
            if observer:
                middlewares.extend(observer.middleware)

        return middlewares

    def register(
        self,
        callback: CallbackType,
        *filters: CallbackType,
        flags: dict[str, Any] | None = None,
        **kwargs: Any,
    ) -> CallbackType:
        """
        Register event handler
        """
        if kwargs:
            msg = (
                "Passing any additional keyword arguments to the registrar method "
                "is not supported.\n"
                "This error may be caused when you are trying to register filters like in 2.x "
                "version of this framework, if it's true just look at correspoding "
                "documentation pages.\n"
                f"Please remove the {set(kwargs.keys())} arguments from this call.\n"
            )
            raise UnsupportedKeywordArgument(msg)

        if flags is None:
            flags = {}

        for item in filters:
            if isinstance(item, Filter):
                item.update_handler_flags(flags=flags)

        self.handlers.append(
            HandlerObject(
                callback=callback,
                filters=[FilterObject(filter_) for filter_ in filters],
                flags=flags,
            ),
        )

        return callback

    def wrap_outer_middleware(
        self,
        callback: Any,
        event: TelegramObject,
        data: dict[str, Any],
    ) -> Any:
        wrapped_outer = self.middleware.wrap_middlewares(
            self.outer_middleware,
            callback,
        )
        return wrapped_outer(event, data)

    def check_root_filters(self, event: TelegramObject, **kwargs: Any) -> Any:
        return self._handler.check(event, **kwargs)

    async def trigger(self, event: TelegramObject, **kwargs: Any) -> Any:
        """
        Propagate event to handlers and stops propagation on first match.
        Handler will be called when all its filters are pass.
        """
        for handler in self.handlers:
            kwargs["handler"] = handler
            result, data = await handler.check(event, **kwargs)
            if result:
                kwargs.update(data)
                try:
                    wrapped_inner = self.outer_middleware.wrap_middlewares(
                        self._resolve_middlewares(),
                        handler.call,
                    )
                    return await wrapped_inner(event, kwargs)
                except SkipHandler:
                    continue

        return UNHANDLED

    def __call__(
        self,
        *filters: CallbackType,
        flags: dict[str, Any] | None = None,
        **kwargs: Any,
    ) -> Callable[[CallbackType], CallbackType]:
        """
        Decorator for registering event handlers
        """

        def wrapper(callback: CallbackType) -> CallbackType:
            self.register(callback, *filters, flags=flags, **kwargs)
            return callback

        return wrapper



from enum import Enum


class TransactionPartnerUserTransactionTypeEnum(str, Enum):
    """
    This object represents type of the transaction that were made by partner user.

    Source: https://core.telegram.org/bots/api#transactionpartneruser
    """

    INVOICE_PAYMENT = "invoice_payment"
    PAID_MEDIA_PAYMENT = "paid_media_payment"
    GIFT_PURCHASE = "gift_purchase"
    PREMIUM_PURCHASE = "premium_purchase"
    BUSINESS_ACCOUNT_TRANSFER = "business_account_transfer"



from typing import TypeAlias

from .input_media_animation import InputMediaAnimation
from .input_media_live_photo import InputMediaLivePhoto
from .input_media_location import InputMediaLocation
from .input_media_photo import InputMediaPhoto
from .input_media_sticker import InputMediaSticker
from .input_media_venue import InputMediaVenue
from .input_media_video import InputMediaVideo

InputPollOptionMediaUnion: TypeAlias = (
    InputMediaAnimation
    | InputMediaLivePhoto
    | InputMediaLocation
    | InputMediaPhoto
    | InputMediaSticker
    | InputMediaVenue
    | InputMediaVideo
)



from __future__ import annotations

from typing import TYPE_CHECKING, Any

from .base import TelegramObject


class EncryptedCredentials(TelegramObject):
    """
    Describes data required for decrypting and authenticating :class:`aiogram.types.encrypted_passport_element.EncryptedPassportElement`. See the `Telegram Passport Documentation <https://core.telegram.org/passport#receiving-information>`_ for a complete description of the data decryption and authentication processes.

    Source: https://core.telegram.org/bots/api#encryptedcredentials
    """

    data: str
    """Base64-encoded encrypted JSON-serialized data with unique user's payload, data hashes and secrets required for :class:`aiogram.types.encrypted_passport_element.EncryptedPassportElement` decryption and authentication"""
    hash: str
    """Base64-encoded data hash for data authentication"""
    secret: str
    """Base64-encoded secret, encrypted with the bot's public RSA key, required for data decryption"""

    if TYPE_CHECKING:
        # DO NOT EDIT MANUALLY!!!
        # This section was auto-generated via `butcher`

        def __init__(
            __pydantic__self__, *, data: str, hash: str, secret: str, **__pydantic_kwargs: Any
        ) -> None:
            # DO NOT EDIT MANUALLY!!!
            # This method was auto-generated via `butcher`
            # Is needed only for type checking and IDE support without any additional plugins

            super().__init__(data=data, hash=hash, secret=secret, **__pydantic_kwargs)



from __future__ import annotations

from typing import TYPE_CHECKING, Any

from .base import TelegramMethod


class SetStickerSetTitle(TelegramMethod[bool]):
    """
    Use this method to set the title of a created sticker set. Returns :code:`True` on success.

    Source: https://core.telegram.org/bots/api#setstickersettitle
    """

    __returning__ = bool
    __api_method__ = "setStickerSetTitle"

    name: str
    """Sticker set name"""
    title: str
    """Sticker set title, 1-64 characters"""

    if TYPE_CHECKING:
        # DO NOT EDIT MANUALLY!!!
        # This section was auto-generated via `butcher`

        def __init__(
            __pydantic__self__, *, name: str, title: str, **__pydantic_kwargs: Any
        ) -> None:
            # DO NOT EDIT MANUALLY!!!
            # This method was auto-generated via `butcher`
            # Is needed only for type checking and IDE support without any additional plugins

            super().__init__(name=name, title=title, **__pydantic_kwargs)



from enum import Enum, auto


class FSMStrategy(Enum):
    """
    FSM strategy for storage key generation.
    """

    USER_IN_CHAT = auto()
    """State will be stored for each user in chat."""
    CHAT = auto()
    """State will be stored for each chat globally without separating by users."""
    GLOBAL_USER = auto()
    """State will be stored globally for each user globally."""
    USER_IN_TOPIC = auto()
    """State will be stored for each user in chat and topic."""
    CHAT_TOPIC = auto()
    """State will be stored for each chat and topic, but not separated by users."""


def apply_strategy(
    strategy: FSMStrategy,
    chat_id: int,
    user_id: int,
    thread_id: int | None = None,
) -> tuple[int, int, int | None]:
    if strategy == FSMStrategy.CHAT:
        return chat_id, chat_id, None
    if strategy == FSMStrategy.GLOBAL_USER:
        return user_id, user_id, None
    if strategy == FSMStrategy.USER_IN_TOPIC:
        return chat_id, user_id, thread_id
    if strategy == FSMStrategy.CHAT_TOPIC:
        return chat_id, chat_id, thread_id

    return chat_id, user_id, None



from __future__ import annotations

from typing import TYPE_CHECKING, Any

from .base import TelegramObject

if TYPE_CHECKING:
    from .chat import Chat
    from .custom import DateTime
    from .user import User


class GiveawayWinners(TelegramObject):
    """
    This object represents a message about the completion of a giveaway with public winners.

    Source: https://core.telegram.org/bots/api#giveawaywinners
    """

    chat: Chat
    """The chat that created the giveaway"""
    giveaway_message_id: int
    """Identifier of the message with the giveaway in the chat"""
    winners_selection_date: DateTime
    """Point in time (Unix timestamp) when winners of the giveaway were selected"""
    winner_count: int
    """Total number of winners in the giveaway"""
    winners: list[User]
    """List of up to 100 winners of the giveaway"""
    additional_chat_count: int | None = None
    """*Optional*. The number of other chats the user had to join in order to be eligible for the giveaway"""
    prize_star_count: int | None = None
    """*Optional*. The number of Telegram Stars that were split between giveaway winners; for Telegram Star giveaways only"""
    premium_subscription_month_count: int | None = None
    """*Optional*. The number of months the Telegram Premium subscription won from the giveaway will be active for; for Telegram Premium giveaways only"""
    unclaimed_prize_count: int | None = None
    """*Optional*. Number of undistributed prizes"""
    only_new_members: bool | None = None
    """*Optional*. :code:`True`, if only users who had joined the chats after the giveaway started were eligible to win"""
    was_refunded: bool | None = None
    """*Optional*. :code:`True`, if the giveaway was canceled because the payment for it was refunded"""
    prize_description: str | None = None
    """*Optional*. Description of additional giveaway prize"""

    if TYPE_CHECKING:
        # DO NOT EDIT MANUALLY!!!
        # This section was auto-generated via `butcher`

        def __init__(
            __pydantic__self__,
            *,
            chat: Chat,
            giveaway_message_id: int,
            winners_selection_date: DateTime,
            winner_count: int,
            winners: list[User],
            additional_chat_count: int | None = None,
            prize_star_count: int | None = None,
            premium_subscription_month_count: int | None = None,
            unclaimed_prize_count: int | None = None,
            only_new_members: bool | None = None,
            was_refunded: bool | None = None,
            prize_description: str | None = None,
            **__pydantic_kwargs: Any,
        ) -> None:
            # DO NOT EDIT MANUALLY!!!
            # This method was auto-generated via `butcher`
            # Is needed only for type checking and IDE support without any additional plugins

            super().__init__(
                chat=chat,
                giveaway_message_id=giveaway_message_id,
                winners_selection_date=winners_selection_date,
                winner_count=winner_count,
                winners=winners,
                additional_chat_count=additional_chat_count,
                prize_star_count=prize_star_count,
                premium_subscription_month_count=premium_subscription_month_count,
                unclaimed_prize_count=unclaimed_prize_count,
                only_new_members=only_new_members,
                was_refunded=was_refunded,
                prize_description=prize_description,
                **__pydantic_kwargs,
            )
